import { __decorate, __param, __metadata } from 'tslib';
import { Injector, Attribute, Component, ViewChild, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { CircleProgressComponent, NgCircleProgressModule } from 'ng-circle-progress';
import { register, PROP_STRING, PROP_ANY, PROP_NUMBER, PROP_BOOLEAN, StylableComponent, styler, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { isString, findValueOf, isDefined } from '@wm/core';

const registerProps = () => {
    register('wm-progress-bar', new Map([
        ['captionplacement', Object.assign({ value: 'hidden' }, PROP_STRING)],
        ['class', PROP_STRING],
        ['dataset', PROP_ANY],
        ['datavalue', PROP_STRING],
        ['displayformat', PROP_STRING],
        ['hint', PROP_STRING],
        ['maxvalue', Object.assign({ value: 100 }, PROP_NUMBER)],
        ['minvalue', Object.assign({ value: 0 }, PROP_NUMBER)],
        ['name', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['tabindex', Object.assign({ value: 0 }, PROP_NUMBER)],
        ['type', Object.assign({ value: 'default' }, PROP_STRING)]
    ]));
};

// This function returns the maximum number of decimal digits allowed.
const getDecimalCount = (val) => {
    val = val || '9';
    val = val.replace(/\%$/, '');
    const n = val.lastIndexOf('.');
    return (n === -1) ? 0 : (val.length - n - 1);
};
// returns true if the given value contains '%'
const isPercentageValue = (val) => {
    if (isString(val)) {
        val = val.trim();
        return val.charAt(val.length - 1) === '%';
    }
    return false;
};
const calculatePercent = (value, min = 0, max = 0) => {
    const percent = ((value - min) / (max - min)) * 100;
    if (_.isNaN(percent)) {
        console.warn('Circle Progress Bar: One of the properties Min, Max or datavalue is not a valid number');
        return 0;
    }
    return Math.abs(percent);
};

var ProgressBarComponent_1;
const DEFAULT_CLS = 'progress app-progress';
const WIDGET_CONFIG = { widgetType: 'wm-progress-bar', hostClass: DEFAULT_CLS };
// map of progress-bar type and classes
const TYPE_CLASS_MAP = {
    'default': '',
    'default-striped': 'progress-bar-striped',
    'success': 'progress-bar-success',
    'success-striped': 'progress-bar-success progress-bar-striped',
    'info': 'progress-bar-info',
    'info-striped': 'progress-bar-info progress-bar-striped',
    'warning': 'progress-bar-warning',
    'warning-striped': 'progress-bar-warning progress-bar-striped',
    'danger': 'progress-bar-danger',
    'danger-striped': 'progress-bar-danger progress-bar-striped'
};
let ProgressBarComponent = ProgressBarComponent_1 = class ProgressBarComponent extends StylableComponent {
    constructor(inj, dataset, boundDataset) {
        super(inj, WIDGET_CONFIG);
        // progress-bar data, ngFor in the template iterates on this
        this.data = [{
                cls: TYPE_CLASS_MAP.default,
                progressBarWidth: '0%',
                displayValue: '0'
            }];
        // flag which determines whether dataset is provided or not
        this.hasDataset = !!(dataset || boundDataset);
        styler(this.nativeElement, this);
        this._prepareData = _.debounce(() => this.prepareData(), 50);
    }
    // update the proper classes when there is a change in type
    onTypeChange() {
        if (!this.hasDataset) {
            if (this.data[0]) {
                this.data[0].cls = TYPE_CLASS_MAP[this.type];
            }
        }
    }
    // returns the formatted display value based on the provided displayformat
    getFormattedDisplayVal(val) {
        const format = this.displayformat || '9';
        val = parseFloat('' + val);
        val = (val.toFixed(getDecimalCount(format)));
        if (format && format.includes('%')) {
            val = `${val}%`;
        }
        return val;
    }
    prepareData() {
        // when the dataset is provided, iterate over the dataset to set the proper values in the data
        if (this.dataset && _.isArray(this.dataset) && this.type && this.datavalue) {
            this.data = this.dataset.map((datum) => {
                const val = findValueOf(datum, this.datavalue).toString();
                let percentVal = val;
                if (val && !val.includes('%')) {
                    percentVal = `${val}%`;
                }
                return {
                    cls: TYPE_CLASS_MAP[findValueOf(datum, this.type)],
                    progressBarWidth: percentVal,
                    displayValue: this.getFormattedDisplayVal(val)
                };
            });
        }
        else {
            // if the dataset is not provided, update the values in the default data
            let width = 0;
            let displayVal = 0;
            if (this.datavalue) {
                if (isPercentageValue(this.datavalue)) {
                    const val = (this.datavalue || '0%');
                    width = displayVal = val;
                }
                else {
                    if (isDefined(this.datavalue)) {
                        const denominator = (+this.maxvalue - +this.minvalue) || 1;
                        const val = ((+this.datavalue - +this.minvalue) * 100) / denominator + '%';
                        width = displayVal = val;
                    }
                }
            }
            this.data[0].displayValue = this.getFormattedDisplayVal(displayVal);
            this.data[0].progressBarWidth = width;
            this.data[0].cls = TYPE_CLASS_MAP[this.type];
        }
    }
    onPropertyChange(key, nv, ov) {
        if (key === 'type') {
            this.onTypeChange();
        }
        else if (key === 'minvalue' || key === 'maxvalue' || key === 'datavalue' || key === 'dataset' || key === 'displayformat') {
            this._prepareData();
        }
        else {
            super.onPropertyChange(key, nv, ov);
        }
    }
};
ProgressBarComponent.initializeProps = registerProps();
ProgressBarComponent.ctorParameters = () => [
    { type: Injector },
    { type: String, decorators: [{ type: Attribute, args: ['dataset',] }] },
    { type: String, decorators: [{ type: Attribute, args: ['dataset.bind',] }] }
];
ProgressBarComponent = ProgressBarComponent_1 = __decorate([
    Component({
        selector: '[wmProgressBar]',
        template: "<div class=\"multi-bar progress-bar {{bar.cls}}\" *ngFor=\"let bar of data\" [style.width]=\"bar.progressBarWidth\" role=\"progressbar\"\n     [attr.aria-valuenow]=\"datavalue\" [attr.aria-valuemin]=\"minvalue\" [attr.aria-valuemax]=\"maxvalue\">\n    <span class=\"app-progress-label\" [attr.data-caption-placement]=\"captionplacement\">{{bar.displayValue}}</span>\n</div>",
        providers: [
            provideAsWidgetRef(ProgressBarComponent_1)
        ]
    }),
    __param(1, Attribute('dataset')), __param(2, Attribute('dataset.bind')),
    __metadata("design:paramtypes", [Injector, String, String])
], ProgressBarComponent);

const registerProps$1 = () => {
    register('wm-progress-circle', new Map([
        ['captionplacement', Object.assign({ value: 'inside' }, PROP_STRING)],
        ['class', PROP_STRING],
        ['dataset', PROP_ANY],
        ['datavalue', Object.assign({ value: '30' }, PROP_STRING)],
        ['displayformat', Object.assign({ value: '9%' }, PROP_STRING)],
        ['hint', PROP_STRING],
        ['maxvalue', Object.assign({ value: 100 }, PROP_NUMBER)],
        ['minvalue', Object.assign({ value: 0 }, PROP_NUMBER)],
        ['name', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['subtitle', PROP_STRING],
        ['tabindex', Object.assign({ value: 0 }, PROP_NUMBER)],
        ['title', PROP_STRING],
        ['type', Object.assign({ value: 'default' }, PROP_STRING)]
    ]));
};

var ProgressCircleComponent_1;
const DEFAULT_CLS$1 = 'progress app-progress circle';
const WIDGET_CONFIG$1 = { widgetType: 'wm-progress-circle', hostClass: DEFAULT_CLS$1 };
const DEFAULT_OPTIONS = {
    responsive: true,
    innerStrokeWidth: 10,
    outerStrokeWidth: 10,
    unitsFontSize: '15',
    space: -10,
    toFixed: 0,
    maxPercent: 100,
    showSubtitle: false,
    clockwise: true,
    startFromZero: false,
    renderOnClick: false,
    innerStrokeColor: '',
    outerStrokeColor: ''
};
// map of progress-bar type and classes
const TYPE_CLASS_MAP$1 = {
    'default': '',
    'success': 'progress-circle-success',
    'info': 'progress-circle-info',
    'warning': 'progress-circle-warning',
    'danger': 'progress-circle-danger',
};
let ProgressCircleComponent = ProgressCircleComponent_1 = class ProgressCircleComponent extends StylableComponent {
    constructor(inj) {
        super(inj, WIDGET_CONFIG$1);
        styler(this.nativeElement, this);
        this.options = _.clone(DEFAULT_OPTIONS);
        this.redraw = _.debounce(this._redraw, 100);
    }
    _redraw() {
        this.circleRef.render();
    }
    ngAfterViewInit() {
        super.ngAfterViewInit();
        this.invokeEventCallback('beforerender', { '$event': {} });
        this.options = Object.assign(this.circleRef.options, this.options);
    }
    overrideDefaults(options) {
        _.extend(this.options, options);
    }
    updateDisplayValueFormat() {
        // show title and subtitle only when captionplacement is 'inside'
        this.options.showTitle = this.options.showSubtitle = (this.captionplacement === 'inside');
        // show units when title value is empty and captionplacement is 'inside'
        this.options.showUnits = !this.title && this.options.showTitle && isPercentageValue(this.displayformat);
        this.options.toFixed = getDecimalCount(this.displayformat);
        if (this.options.showTitle) {
            this.options.title = this.title || 'auto';
            this.options.showSubtitle = !!this.subtitle;
            this.options.subtitle = this.subtitle || '';
        }
        this.redraw();
    }
    onPropertyChange(key, nv, ov) {
        switch (key) {
            case 'type':
                this.$element.removeClass(TYPE_CLASS_MAP$1[ov]);
                this.$element.addClass(TYPE_CLASS_MAP$1[nv]);
                break;
            case 'minvalue':
            case 'maxvalue':
            case 'datavalue':
                if (isPercentageValue(this.datavalue)) {
                    this.percentagevalue = parseFloat(this.datavalue);
                }
                else {
                    this.percentagevalue = calculatePercent(parseFloat(this.datavalue), this.minvalue, this.maxvalue);
                }
                break;
            case 'displayformat':
            case 'captionplacement':
            case 'title':
            case 'subtitle':
                this.updateDisplayValueFormat();
                break;
        }
        super.onPropertyChange(key, nv, ov);
    }
};
ProgressCircleComponent.initializeProps = registerProps$1();
ProgressCircleComponent.ctorParameters = () => [
    { type: Injector }
];
__decorate([
    ViewChild(CircleProgressComponent, { static: true }),
    __metadata("design:type", CircleProgressComponent)
], ProgressCircleComponent.prototype, "circleRef", void 0);
ProgressCircleComponent = ProgressCircleComponent_1 = __decorate([
    Component({
        selector: '[wmProgressCircle]',
        template: "<circle-progress [percent]=\"percentagevalue\" [options]=\"options\"></circle-progress>\n",
        providers: [
            provideAsWidgetRef(ProgressCircleComponent_1)
        ]
    }),
    __metadata("design:paramtypes", [Injector])
], ProgressCircleComponent);

const components = [
    ProgressBarComponent,
    ProgressCircleComponent
];
let ProgressModule = class ProgressModule {
};
ProgressModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            NgCircleProgressModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], ProgressModule);

/**
 * Generated bundle index. Do not edit.
 */

export { ProgressModule, ProgressBarComponent as ɵa, registerProps as ɵb, ProgressCircleComponent as ɵc, registerProps$1 as ɵd };
//# sourceMappingURL=index.js.map
