import { __decorate, __metadata, __param } from 'tslib';
import { Injector, Attribute, ViewChild, ElementRef, ViewChildren, QueryList, Component, Directive, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { NG_VALUE_ACCESSOR, NG_VALIDATORS, FormsModule } from '@angular/forms';
import { TypeaheadDirective, TypeaheadModule } from 'ngx-bootstrap/typeahead';
import { PROP_BOOLEAN, PROP_STRING, PROP_ANY, PROP_NUMBER, register, registerFormWidget, interpolateBindExpressions, ALLFIELDS, extractDataAsArray, convertDataToObject, transformFormData, getUniqObjsByDataField, styler, getContainerTargetClass, provideAs, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { Observable, of, from } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
import { FormWidgetType, DataSource, getClonedObject, AppConstants, findValueOf, addClass, isDefined, isMobile, adjustContainerPosition, toBoolean } from '@wm/core';
import { DatasetAwareFormComponent } from '@wm/components/input';

const searchProps = new Map([
    ['casesensitive', Object.assign({ value: false }, PROP_BOOLEAN)],
    ['class', PROP_STRING],
    ['datacompletemsg', Object.assign({ value: 'No more data to load' }, PROP_STRING)],
    ['datafield', Object.assign({ value: 'All Fields' }, PROP_STRING)],
    ['dataoptions', PROP_ANY],
    ['dataset', PROP_ANY],
    ['datasource', PROP_ANY],
    ['datavalue', PROP_STRING],
    ['datavaluesource', PROP_ANY],
    ['disabled', PROP_BOOLEAN],
    ['displayimagesrc', PROP_STRING],
    ['displaylabel', PROP_STRING],
    ['displayexpression', PROP_STRING],
    ['dropup', PROP_BOOLEAN],
    ['hint', PROP_STRING],
    ['imagewidth', Object.assign({ value: '16px' }, PROP_STRING)],
    ['limit', PROP_NUMBER],
    ['loadingdatamsg', Object.assign({ value: 'Loading items...' }, PROP_STRING)],
    ['matchmode', PROP_STRING],
    ['minchars', PROP_NUMBER],
    ['name', PROP_STRING],
    ['navsearchbar', Object.assign({ value: false }, PROP_BOOLEAN)],
    ['orderby', PROP_STRING],
    ['placeholder', Object.assign({ value: 'Search' }, PROP_STRING)],
    ['position', PROP_STRING],
    ['query', PROP_STRING],
    ['readonly', PROP_BOOLEAN],
    ['required', PROP_BOOLEAN],
    ['searchkey', PROP_STRING],
    ['searchon', Object.assign({ value: 'typing' }, PROP_STRING)],
    ['shortcutkey', PROP_STRING],
    ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
    ['showsearchicon', PROP_BOOLEAN],
    ['tabindex', Object.assign({ value: 0 }, PROP_NUMBER)],
    ['type', Object.assign({ value: 'search' }, PROP_STRING)],
    ['debouncetime', Object.assign({ value: 250 }, PROP_NUMBER)],
    ['width', PROP_STRING]
]);
const registerProps = () => {
    register('wm-search', searchProps);
    registerFormWidget(FormWidgetType.AUTOCOMPLETE, new Map(searchProps));
    registerFormWidget(FormWidgetType.TYPEAHEAD, new Map(searchProps));
};

class LocalDataProvider {
    applyFilter(entry, queryText, filtername) {
        entry = _.isNumber(entry) ? entry.toString() : entry;
        if (_.includes(filtername, 'start')) {
            return _.startsWith(entry, queryText);
        }
        else if (_.includes(filtername, 'end')) {
            return _.endsWith(entry, queryText);
        }
        else if (_.includes(filtername, 'exact')) {
            return _.isEqual(entry, queryText);
        }
        return _.includes(entry, queryText);
    }
    // LocalData filtering is done based on the searchkey.
    filter(config) {
        const entries = config.dataset;
        let casesensitive = config.casesensitive;
        const matchMode = config.matchMode;
        // for supporting existing projects
        if (matchMode && !_.includes(matchMode, 'ignorecase')) {
            casesensitive = true;
        }
        let queryText = config.query, filteredData;
        return new Promise((resolve, reject) => {
            /**
             * If searchKey is defined, then check for match string against each item in the dataset with item's field name as the searchKey
             * return the filtered data containing the matching string.
             */
            if (config.searchKey) {
                const keys = _.split(config.searchKey, ',');
                filteredData = _.filter(config.dataset, (item) => {
                    return keys.some(key => {
                        let a = _.get(item, key), b = queryText;
                        if (!casesensitive) {
                            a = _.toLower(_.toString(a));
                            b = _.toLower(_.toString(b));
                        }
                        return this.applyFilter(a, b, matchMode);
                    });
                });
            }
            else {
                // local search on data with array of objects.
                // Iterate over each item and return the filtered data containing the matching string.
                if (_.isArray(entries) && _.isObject(entries[0])) {
                    filteredData = _.filter(entries, entry => {
                        let a = _.values(entry).join(' ');
                        if (!casesensitive) {
                            a = _.toLower(a);
                            queryText = _.toLower(queryText);
                        }
                        return this.applyFilter(a, queryText);
                    });
                }
                else {
                    filteredData = _.filter(entries, entry => {
                        if (!casesensitive) {
                            entry = _.toLower(entry);
                            queryText = _.toLower(queryText);
                        }
                        return this.applyFilter(entry, queryText, matchMode);
                    });
                }
            }
            resolve({
                data: filteredData,
                hasMoreData: false,
                isLastPage: true
            });
        });
    }
}

class RemoteDataProvider {
    filter(config) {
        return this.filterData(config).then(response => this.onFilterSuccess(config, response), () => this.onFilterFailure());
    }
    filterData(config) {
        if (config.dataoptions) {
            const dataoptions = config.dataoptions;
            const requestParams = config.datasource.execute(DataSource.Operation.GET_REQUEST_PARAMS, config);
            // If options are specified, make specifics calls to fetch the results
            // Fetch the related field data
            if (dataoptions.relatedField) {
                return new Promise((resolve, reject) => {
                    interpolateBindExpressions(config.viewParent, dataoptions.filterExpr, (filterexpressions) => {
                        requestParams.filterExpr = dataoptions.filterExpr = filterexpressions;
                        dataoptions.filterExpr = filterexpressions;
                        config.datasource.execute(DataSource.Operation.GET_RELATED_TABLE_DATA, _.assign({ relatedField: dataoptions.relatedField }, requestParams)).then(resolve, reject);
                    });
                });
            }
            // Fetch the distinct data
            if (dataoptions.distinctField) {
                return config.datasource.execute(DataSource.Operation.GET_DISTINCT_DATA_BY_FIELDS, {
                    pagesize: config.limit || config.pagesize,
                    page: config.page,
                    fields: dataoptions.distinctField,
                    entityName: dataoptions.tableName,
                    filterFields: _.assign(dataoptions.filterFields, requestParams.filterFields),
                    filterExpr: getClonedObject(dataoptions.filterExpr || {})
                });
            }
        }
        // search records using the searchkey
        return config.datasource.execute(DataSource.Operation.SEARCH_RECORDS, config);
    }
    // Check if the page retrieved currently is the last page. If last page, don't send any more request
    isLast(page, dataSize, maxResults, currentResults) {
        // if last page info is not returned by backend and current results is less than max results, this is the last page
        if (dataSize === AppConstants.INT_MAX_VALUE) {
            return currentResults !== 0 && currentResults < maxResults;
        }
        const pageCount = ((dataSize > maxResults) ? (Math.ceil(dataSize / maxResults)) : (dataSize < 0 ? 0 : 1));
        return page === pageCount;
    }
    // this function transform the response data in case it is not an array
    getTransformedData(variable, data) {
        const operationResult = variable.operation + 'Result'; // when output is only string it is available as oprationNameResult
        const tempResponse = data[operationResult];
        // in case data received is value as string then add that string value to object and convert object into array
        if (tempResponse) {
            const tempObj = {};
            _.set(tempObj, operationResult, tempResponse);
            data = [tempObj]; // convert data into an array having tempObj
        }
        else {
            // in case data received is already an object then convert it into an array
            data = [data];
        }
        return data;
    }
    onFilterFailure() {
        return [];
    }
    isLastPageForDistinctApi(data, page, totalElements, _isLastPage) {
        return page > 1 && !_isLastPage && _.isEmpty(data) && totalElements === AppConstants.INT_MAX_VALUE;
    }
    // this function processes the response depending on pageOptions, isPageable and prepares the formattedDataset.
    onFilterSuccess(config, response) {
        let data = response.data;
        let formattedData;
        let _isLastPage;
        let page;
        let isPaginatedData;
        const expressionArray = _.split(config.binddataset, '.');
        const dataExpression = _.slice(expressionArray, _.indexOf(expressionArray, 'dataSet') + 1).join('.');
        const $I = '[$i]';
        return new Promise((resolve, reject) => {
            const pageOptions = response.pagination;
            if (config.datasource.execute(DataSource.Operation.IS_PAGEABLE)) {
                page = pageOptions.number + 1;
                _isLastPage = this.isLast(page, (config.limit > 0 && config.limit) || pageOptions.totalElements, pageOptions.size, pageOptions.numberOfElements);
                isPaginatedData = true;
                if (this.isLastPageForDistinctApi(data, page, pageOptions.totalElements, _isLastPage)) {
                    _isLastPage = true;
                    resolve({
                        data: [],
                        isLastPage: _isLastPage,
                        hasMoreData: page > 1,
                        isPaginatedData,
                        page
                    });
                    return;
                }
            }
            // if data expression exists, extract the data from the expression path
            if (dataExpression) {
                if (_.isArray(data)) {
                    const index = dataExpression.lastIndexOf($I);
                    if (index > -1) {
                        const restExpr = dataExpression.substr(index + 5);
                        formattedData = data.map(datum => findValueOf(datum, restExpr));
                    }
                }
                else if (_.isObject(data)) {
                    formattedData = _.get(data, dataExpression);
                }
                data = formattedData || data;
            }
            if (!_.isArray(data)) {
                data = this.getTransformedData(config.datasource, data);
            }
            // in case of no data received, resolve the promise with empty array
            if (!data.length) {
                resolve({ data: [], isLastPage: _isLastPage, hasMoreData: page > 1, isPaginatedData, page });
            }
            else {
                resolve({ data: data, isLastPage: _isLastPage, hasMoreData: page > 1, isPaginatedData, page });
            }
        });
    }
}

class DataProvider {
    filter(config) {
        let promise;
        /**
         * Make call to remoteDataProvider when searchkey is available and data is not from local / model variable.
         * Otherwise use localDataProvider
         * If datasource is a serviceVariable with no input params, then perform local search.
         * when there is no dataset on the datasource when first time make a remote call to set the dataset for service variable.
         */
        const hasNoVariableDataset = config.datasource && config.datasource.execute(DataSource.Operation.IS_UPDATE_REQUIRED, config.hasData);
        if (!config.isLocalFilter && (config.dataoptions || ((config.datasource && config.datasource.execute(DataSource.Operation.IS_API_AWARE))
            && config.searchKey
            && hasNoVariableDataset))) {
            promise = DataProvider.remoteDataProvider.filter(config);
        }
        else {
            promise = DataProvider.localDataProvider.filter(config);
        }
        return promise.then(response => {
            this.updateDataset = config.datasource && !config.datasource.execute(DataSource.Operation.SUPPORTS_CRUD) && hasNoVariableDataset;
            this.hasMoreData = response.hasMoreData;
            this.isLastPage = response.isLastPage;
            this.page = response.page;
            this.isPaginatedData = response.isPaginatedData;
            return response;
        });
    }
}
DataProvider.remoteDataProvider = new RemoteDataProvider();
DataProvider.localDataProvider = new LocalDataProvider();

var SearchComponent_1;
const WIDGET_CONFIG = { widgetType: 'wm-search', hostClass: 'input-group' };
let SearchComponent = SearchComponent_1 = class SearchComponent extends DatasetAwareFormComponent {
    constructor(inj, binddatavalue, binddataset) {
        super(inj, WIDGET_CONFIG);
        this.binddatavalue = binddatavalue;
        this.binddataset = binddataset;
        this.query = '';
        this.page = 1;
        // this flag will not allow the empty datafield values.
        this.allowempty = false;
        addClass(this.nativeElement, 'app-search', true);
        /**
         * Listens for the change in the ngModel on every search and retrieves the data as observable.
         * This observable data is passed to the typeahead.
         * @type {Observable<any>}
         */
        this.typeaheadDataSource = Observable
            .create((observer) => {
            // Runs on every search
            if (this.listenQuery) {
                this._defaultQueryInvoked = false;
                this._loadingItems = true;
                observer.next(this.query);
            }
            // on keydown, while scrolling the dropdown items, when last item is reached next call is triggered
            // unless the call is resolved, we are able to scroll next to first item and soon
            // This shows flickering from first item to next new items appended.
            // By setting container to undefined, key events changes will be stopped while loading items
            if (this.lastSelectedIndex) {
                this.typeahead._container = undefined;
            }
        }).pipe(mergeMap((token) => this.getDataSourceAsObservable(token)));
        this.dataProvider = new DataProvider();
        /**
         * When default datavalue is not found within the dataset, a filter call is made to get the record using fetchDefaultModel.
         * after getting the response, set the queryModel and query.
         */
        const datavalueSubscription = this.datavalue$.subscribe((val) => {
            const query = (_.isArray(val) ? val[0] : val);
            if (query === null || query === '') {
                this._modelByValue = '';
                // reset the query.
                this.query = this.queryModel = '';
                // on clear or reset filter, empty the lastResults to fetch new records.
                this._lastResult = undefined;
                return;
            }
            if (!this._unsubscribeDv) {
                this._defaultQueryInvoked = false;
                // if prev datavalue is not equal to current datavalue then clear the modelByKey and queryModel
                if (!_.isObject(val) && this.prevDatavalue !== val) {
                    this._modelByKey = undefined;
                    this.query = this.queryModel = '';
                }
                // if the datafield is ALLFILEDS do not fetch the records
                // update the query model with the values we have
                this.updateByDatavalue(val);
            }
        });
        this.registerDestroyListener(() => datavalueSubscription.unsubscribe());
        const datasetSubscription = this.dataset$.subscribe(() => {
            // set the next item index.
            this.startIndex = this.datasetItems.length;
            this._lastResult = undefined;
            const defaultValue = this.datavalue || this.toBeProcessedDatavalue;
            // invoking updateByDataset only when datavalue is present, as entered value i.e. query is setting back to ''
            if (isDefined(defaultValue)) {
                this.updateByDataset(defaultValue);
            }
        });
        this.registerDestroyListener(() => datasetSubscription.unsubscribe());
    }
    // getter setter is added to pass the datasource to searchcomponent.
    get datasource() {
        return this._datasource;
    }
    set datasource(nv) {
        this._datasource = nv;
        const data = this.datavalue || this.toBeProcessedDatavalue;
        this.updateByDatavalue(data);
    }
    // on clear, trigger search with page size 1
    clearSearch($event, loadOnClear) {
        this.query = '';
        this.onInputChange($event);
        this.dataProvider.isLastPage = false;
        this.listenQuery = false;
        if (loadOnClear) {
            this.listenQuery = true;
            this._unsubscribeDv = false;
            this.loadMoreData();
        }
        this.invokeEventCallback('clearsearch');
    }
    // function to  clear the input value
    clearText() {
        this.query = '';
        this.showClosebtn = false;
    }
    // Close the full screen mode in mobile view of auto complete
    closeSearch() {
        this._loadingItems = false;
        this.page = 1;
        // after closing the search, insert the element at its previous position (elIndex)
        this.insertAtIndex(this.elIndex);
        this.elIndex = undefined;
        this.parentEl = undefined;
        this.$element.removeClass('full-screen');
        if (this._domUpdated) {
            this._domUpdated = false;
        }
        this.listenQuery = false;
        this._unsubscribeDv = true;
        this.typeahead.hide();
    }
    renderMobileAutoComplete() {
        // Get the parent element of the search element which can be next or prev element, if both are empty then get the parent of element.
        if (!isDefined(this.elIndex)) {
            this.parentEl = this.$element.parent();
            this.elIndex = this.parentEl.children().index(this.$element);
        }
        if (!this.$element.hasClass('full-screen')) {
            // this flag is set to notify that the typeahead-container dom has changed its position
            this._domUpdated = true;
            // Add full screen class on focus of the input element.
            this.$element.addClass('full-screen');
            // Add position to set the height to auto
            if (this.position === 'inline') {
                this.$element.addClass(this.position);
            }
        }
        // focus is lost when element is changed to full-screen, keydown to select next items will not work
        // Hence explicitly focusing the input
        if (this.$element.hasClass('full-screen')) {
            this.$element.find('.app-search-input').focus();
        }
    }
    getDataSourceAsObservable(query) {
        // show dropdown only when there is change in query. This should not apply when dataoptions with filterFields are updated.
        // when lastResult is not available i.e. still the first call is pending and second query is invoked then do not return.
        if (this._lastQuery === query && !_.get(this.dataoptions, 'filterFields') && isDefined(this._lastResult)) {
            this._loadingItems = false;
            return of(this._lastResult);
        }
        /*
             WMS-19177: On focus of input field for autocomplete type, do not trigger api call when
                 a) searchkey and displayexpression are different
                 b) binddisplaylabel contains multiple values
                 c) binddisplaylabel contains single value which does not match with searchkey
        */
        if (this.type === 'autocomplete' && this.searchkey && this.query && this.datavalue) {
            let bindDisplayLabelArray = [];
            const searchKeyArray = _.split(this.searchkey, ',');
            let displaylabel;
            if (this.binddisplaylabel) {
                bindDisplayLabelArray = _.split(this.binddisplaylabel, '+');
                if (bindDisplayLabelArray.length === 1) {
                    if (!_.includes(bindDisplayLabelArray[0], '|')) {
                        const regex = /\[(.*?)\]/;
                        displaylabel = regex.exec(bindDisplayLabelArray[0])[1];
                    }
                    else {
                        displaylabel = true;
                    }
                }
            }
            if ((bindDisplayLabelArray.length > 1 || (displaylabel && !_.includes(searchKeyArray, displaylabel))
                || (this.displayexpression && !_.includes(searchKeyArray, this.displayexpression)))) {
                this._loadingItems = false;
                return of([]);
            }
        }
        this._lastQuery = this.query;
        return from(this.getDataSource(query));
    }
    handleEvent(node, eventName, eventCallback, locals) {
        if (!_.includes(['blur', 'focus', 'select', 'submit', 'change'], eventName)) {
            super.handleEvent(node, eventName, eventCallback, locals);
        }
    }
    // highlight the characters in the dropdown matching the query.
    highlight(match, query) {
        if (this.typeaheadContainer) {
            // highlight of chars will work only when label are strings.
            match.value = match.item.label.toString();
            return this.typeaheadContainer.highlight(match, query);
        }
    }
    // inserts the element at the index position
    insertAtIndex(i) {
        if (i === 0) {
            this.parentEl.prepend(this.$element);
        }
        else {
            const $elAtIndex = this.parentEl.children().eq(i);
            if ($elAtIndex.length) {
                this.$element.insertBefore(this.parentEl.children().eq(i));
            }
            else {
                this.$element.insertAfter(this.parentEl.children().eq(i - 1));
            }
        }
    }
    // Check if the widget is of type autocomplete in mobile view/ app
    isMobileAutoComplete() {
        return this.type === 'autocomplete' && isMobile();
    }
    loadMoreData(incrementPage) {
        if (this.dataProvider.isLastPage) {
            return;
        }
        // Increase the page number and trigger force query update
        this.page = incrementPage ? this.page + 1 : this.page;
        this.isScrolled = true;
        this._loadingItems = true;
        // when invoking new set of results, reset the lastQuery.
        if (incrementPage) {
            this._lastQuery = undefined;
        }
        // trigger the typeahead change manually to fetch the next set of results.
        this.typeahead.onInput({
            target: {
                value: _.trim(this.query) || '0' // dummy data to notify the observables
            }
        });
    }
    // on focusout, subscribe to the datavalue changes again
    onFocusOut() {
        this._unsubscribeDv = false;
        this._loadingItems = false;
        // reset the page value on focusout.
        this.page = 1;
        // if domUpdated is true then do not hide the dropdown in the fullscreen
        if (!this._domUpdated && this._isOpen) {
            this.listenQuery = false;
            // hide the typeahead only after the item is selected from dropdown.
            setTimeout(() => {
                if (this.typeahead._typeahead.isShown) {
                    this.typeahead.hide();
                }
            }, 200);
        }
        this._isOpen = false;
        // on outside click, typeahead is hidden. To avoid this, when fullscreen is set, overridding isFocused flag on the typeahead container
        if (this._domUpdated && this.typeahead && this.typeahead._container) {
            this.typeahead._container.isFocused = true;
        }
    }
    onInputChange($event) {
        // reset all the previous page details in order to fetch new set of result.
        this.result = [];
        this.page = 1;
        this.listenQuery = this.isUpdateOnKeyPress();
        this._modelByValue = undefined;
        // when input is cleared, reset the datavalue
        if (this.query === '') {
            this.queryModel = '';
            this._modelByValue = '';
            this.invokeOnChange(this._modelByValue, {}, true);
            // trigger onSubmit only when the search input is cleared off and do not trigger when tab is pressed.
            if ($event && $event.which !== 9) {
                this.invokeEventCallback('submit', { $event });
            }
        }
        else {
            // invoking change event on every input value change.
            this.invokeEventCallback('change', {
                $event: $event,
                newVal: this._modelByValue || this.query,
                oldVal: this.prevDatavalue
            });
        }
        this.showClosebtn = (this.query !== '');
    }
    // Triggered for enter event
    handleEnterEvent($event) {
        // submit event triggered when there is no search results
        if (!this.typeahead._container) {
            this.onSearchSelect($event);
        }
    }
    // Triggerred when typeahead option is selected.
    onSearchSelect($event) {
        let item;
        if (this.typeaheadContainer && this.typeaheadContainer.active) {
            item = this.typeaheadContainer.active.item;
        }
        $event = this.eventData($event, item || {});
        // searchOn is set as onBtnClick, then invoke the search api call manually.
        if (!this.isUpdateOnKeyPress()) {
            this.listenQuery = true;
            // trigger the typeahead change manually to fetch the next set of results.
            this.typeahead.onInput({
                target: {
                    value: this.query // dummy data to notify the observables
                }
            });
            return;
        }
        // when matches are available.
        if (this.typeaheadContainer && this.liElements.length) {
            this.typeaheadContainer.selectActiveMatch();
        }
        else {
            this.queryModel = this.query;
            this.invokeEventCallback('submit', { $event });
        }
    }
    invokeOnBeforeServiceCall(inputData) {
        this.invokeEventCallback('beforeservicecall', { inputData });
    }
    onDropdownOpen() {
        // setting the ulElements, liElement on typeaheadContainer.
        // as we are using customOption template, liElements are not available on typeaheadContainer so append them explicitly.
        const fn = _.debounce(() => {
            this._isOpen = true;
            this.typeaheadContainer = this.typeahead._container || this.typeahead._typeahead.instance;
            this.typeaheadContainer.liElements = this.liElements;
            this.typeaheadContainer.ulElement = this.ulElement;
            adjustContainerPosition($('typeahead-container'), this.nativeElement, this.typeahead._typeahead, $('typeahead-container .dropdown-menu'));
        });
        fn();
        // open full-screen search view
        if (this.isMobileAutoComplete()) {
            if (!this.$element.hasClass('full-screen')) {
                this.renderMobileAutoComplete();
            }
            const dropdownEl = this.dropdownEl.closest('typeahead-container');
            dropdownEl.insertAfter(this.$element.find('input').first());
            const screenHeight = this.$element.closest('.app-content').height();
            dropdownEl.css({ position: 'relative', top: 0, height: screenHeight + 'px' });
            this.showClosebtn = this.query && this.query !== '';
            if (!_.isUndefined(this.dataProvider.isLastPage) && !this.dataProvider.isLastPage) {
                this.triggerSearch();
            }
        }
    }
    selectNext() {
        const matches = this.typeaheadContainer.matches;
        if (!matches) {
            return;
        }
        const index = matches.indexOf(this.typeaheadContainer.active);
        // on keydown, if scroll is at the bottom and next page records are available, fetch next page items.
        if (!this._loadingItems && !this.dataProvider.isLastPage && index + 1 > matches.length - 1) {
            // index is saved in order to select the lastSelected item in the dropdown after fetching next page items.
            this.lastSelectedIndex = index;
            this.loadMoreData(true);
        }
    }
    setLastActiveMatchAsSelected() {
        if (this.lastSelectedIndex) {
            this.typeaheadContainer._active = this.typeaheadContainer.matches[this.lastSelectedIndex];
            this.typeaheadContainer.nextActiveMatch();
            this.lastSelectedIndex = undefined;
        }
    }
    triggerSearch() {
        if (this.dataProvider.isLastPage || !this.$element.hasClass('full-screen')) {
            return;
        }
        const typeAheadDropDown = this.dropdownEl;
        const $lastItem = typeAheadDropDown.find('li').last();
        // Check if last item is not below the full screen
        if ($lastItem.length && typeAheadDropDown.length && (typeAheadDropDown.height() + typeAheadDropDown.position().top > $lastItem.height() + $lastItem.position().top)) {
            this.loadMoreData(true);
        }
    }
    isUpdateOnKeyPress() {
        return this.searchon === 'typing';
    }
    debounceDefaultQuery(data) {
        this._defaultQueryInvoked = true;
        this.getDataSource(data, true).then((response) => {
            if (response.length) {
                this.queryModel = response;
                this._lastQuery = this.query = this.queryModel[0].label || '';
                this._modelByValue = this.queryModel[0].value;
                this._modelByKey = this.queryModel[0].key;
            }
            else {
                this._modelByValue = undefined;
                this.queryModel = undefined;
                this.query = '';
            }
        });
    }
    updateByDatavalue(data) {
        this.updateByDataset(data);
        this.updateByDataSource(data);
    }
    updateByDataSource(data) {
        // value is present but the corresponding key is not found then fetch next set
        // modelByKey will be set only when datavalue is available inside the localData otherwise make a N/w call.
        if (isDefined(data) && !_.isObject(data) && this.datasource && !isDefined(this._modelByKey) && this.datafield !== ALLFIELDS) {
            // Avoid making default query if queryModel already exists.
            if (isDefined(this.queryModel) && !_.isEmpty(this.queryModel)) {
                this.updateDatavalueFromQueryModel();
                return;
            }
            // Make default query call only when datasource supports CRUD (live variable).
            if (!this._defaultQueryInvoked && this.datasource.execute(DataSource.Operation.SUPPORTS_CRUD)) {
                this.debounceDefaultQuery(data);
            }
        }
    }
    // updates the model value using queryModel
    updateDatavalueFromQueryModel() {
        if (isDefined(this.queryModel)) {
            this._modelByValue = _.isArray(this.queryModel) ? this.queryModel[0].value : this.queryModel;
            this._modelByKey = _.isArray(this.queryModel) ? this.queryModel[0].key : this.queryModel;
            this.toBeProcessedDatavalue = undefined;
        }
    }
    updateByDataset(data) {
        // default query is already invoked then do not make other default query call.
        // For local search i.e. searchkey is undefined, do not return, verify the datavalue against the datasetItems .
        if (this._defaultQueryInvoked && this.searchkey) {
            return;
        }
        const selectedItem = _.find(this.datasetItems, (item) => {
            return (_.isObject(item.value) ? _.isEqual(item.value, data) : (_.toString(item.value)).toLowerCase() === (_.toString(data)).toLowerCase());
        });
        // set the default only when it is available in dataset.
        if (selectedItem) {
            this.queryModel = [selectedItem];
        }
        else if (this.datafield === ALLFIELDS && _.isObject(data)) {
            this.queryModel = this.getTransformedData(extractDataAsArray(data));
        }
        else {
            // resetting the queryModel only when prevDatavalue is equal to data
            if (this.prevDatavalue !== data) {
                this.queryModel = undefined;
                this.query = '';
                return;
            }
        }
        this.updateDatavalueFromQueryModel();
        // Show the label value on input.
        this._lastQuery = this.query = isDefined(this.queryModel) && this.queryModel.length ? _.get(this.queryModel[0], 'label') : '';
    }
    // If we have last search results then open dropdown on focus
    handleFocus($event) {
        if (this.type === 'search' && this.query === this._lastQuery && this._lastResult) {
            this.typeahead.keyUpEventEmitter.emit(this.query);
        }
    }
    // This method returns a promise that provides the filtered data from the datasource.
    getDataSource(query, searchOnDataField, nextItemIndex) {
        // For default query, searchOnDataField is set to true, then do not make a n/w call when datafield is ALLFIELDS
        if (searchOnDataField && this.datafield === ALLFIELDS) {
            this._loadingItems = false;
            return Promise.resolve([]);
        }
        // For default datavalue, search key as to be on datafield to get the default data from the filter call.
        const dataConfig = {
            dataset: this.dataset ? convertDataToObject(this.dataset) : undefined,
            binddataset: this.binddataset,
            datasource: this.datasource,
            datafield: this.datafield,
            hasData: this.dataset && this.dataset.length,
            query: query,
            isLocalFilter: !this.searchkey,
            searchKey: searchOnDataField ? this.datafield : this.searchkey,
            // default search call match mode should be startignorecase
            matchMode: searchOnDataField ? 'startignorecase' : this.matchmode,
            casesensitive: this.casesensitive,
            isformfield: this.isformfield,
            orderby: this.orderby,
            limit: this.limit,
            pagesize: this.pagesize,
            page: this.page,
            onBeforeservicecall: this.invokeOnBeforeServiceCall.bind(this)
        };
        if (this.dataoptions) {
            dataConfig.dataoptions = this.dataoptions;
            dataConfig.viewParent = this.viewParent;
        }
        this._loadingItems = true;
        return this.dataProvider.filter(dataConfig)
            .then((response) => {
            // response from dataProvider returns always data object.
            response = response.data || response;
            // for service variable, updating the dataset only if it is not defined or empty
            if ((!isDefined(this.dataset) || !this.dataset.length) && this.dataProvider.updateDataset) {
                this.dataset = response;
            }
            if (this.dataProvider.hasMoreData) {
                this.formattedDataset = this.formattedDataset.concat(response);
            }
            else {
                this.formattedDataset = response;
            }
            // explicitly setting the optionslimit as the matches more than 20 will be ignored if optionslimit is not specified.
            if (this.formattedDataset.length > 20 && !isDefined(this.limit)) {
                this.typeahead.typeaheadOptionsLimit = this.formattedDataset.length;
            }
            // In mobile, trigger the search by default until the results have height upto page height. Other results can be fetched by scrolling
            if (this._isOpen && this.isMobileAutoComplete() && !_.isUndefined(this.dataProvider.isLastPage) && !this.dataProvider.isLastPage) {
                this.triggerSearch();
            }
            const transformedData = this.getTransformedData(this.formattedDataset, nextItemIndex);
            // result contains the datafield values.
            this.result = _.map(transformedData, 'value');
            return transformedData;
        }, (error) => {
            this._loadingItems = false;
            return [];
        }).then(result => {
            if (this.isScrolled) {
                (_.debounce(() => {
                    this.setLastActiveMatchAsSelected();
                }, 30))();
                this.isScrolled = false;
            }
            // When no result is found, set the datavalue to undefined.
            if (!result.length) {
                this._modelByValue = undefined;
                this.queryModel = query;
            }
            // on focusout i.e. on other widget focus, if n/w is pending loading icon is shown, when data is available then dropdown is shown again.
            // on unsubscribing do not show the results.
            if (this._unsubscribeDv) {
                result = [];
            }
            this._loadingItems = false;
            this._lastResult = result;
            return result;
        });
    }
    // this method returns the parent context i.e. Page context and not parent component's context.
    getContext(context) {
        if (context && _.get(context, 'widget')) {
            return this.getContext(context.viewParent);
        }
        return context;
    }
    getTransformedData(data, itemIndex, iscustom) {
        if (isDefined(itemIndex)) {
            itemIndex++;
        }
        const transformedData = transformFormData(this.getContext(this.viewParent), data, this.datafield, {
            displayField: this.displaylabel || this.displayfield,
            displayExpr: iscustom ? '' : this.displayexpression,
            bindDisplayExpr: iscustom ? '' : this.binddisplaylabel,
            bindDisplayImgSrc: this.binddisplayimagesrc,
            displayImgSrc: this.displayimagesrc
        }, itemIndex);
        return getUniqObjsByDataField(transformedData, this.datafield, this.displayfield || this.displaylabel, toBoolean(this.allowempty));
    }
    // OptionsListTemplate listens to the scroll event and triggers this function.
    onScroll($scrollEl, evt) {
        const totalHeight = $scrollEl.scrollHeight, clientHeight = $scrollEl.clientHeight;
        // If scroll is at the bottom and no request is in progress and next page records are available, fetch next page items.
        if (!this._loadingItems && !this.dataProvider.isLastPage && ($scrollEl.scrollTop + clientHeight >= totalHeight)) {
            this.loadMoreData(true);
        }
    }
    ngOnInit() {
        super.ngOnInit();
        if (!isDefined(this.minchars)) {
            // for autocomplete set the minchars to 0
            if (this.type === 'autocomplete') {
                this.minchars = 0;
            }
            else {
                this.minchars = 1;
            }
        }
        this.listenQuery = this.isUpdateOnKeyPress();
        // by default for autocomplete do not show the search icon
        // by default show the searchicon for type = search
        this.showsearchicon = isDefined(this.showsearchicon) ? this.showsearchicon : (this.type === 'search');
    }
    ngAfterViewInit() {
        super.ngAfterViewInit();
        styler(this.nativeElement, this);
        this.containerTarget = getContainerTargetClass(this.nativeElement);
    }
    eventData($event, item) {
        if ($event) {
            $event['data'] = {
                item: item.dataObject,
                model: item.value,
                label: item.label,
                query: this.query
            };
        }
        return $event;
    }
    // triggered on select on option from the list. Set the queryModel, query and modelByKey from the matched item.
    typeaheadOnSelect(match, $event) {
        const item = match.item;
        this.queryModel = item;
        item.selected = true;
        this.query = item.label;
        $event = $event || this.$typeaheadEvent;
        // As item.key can vary from key in the datasetItems
        this._modelByKey = item.key;
        this._modelByValue = item.value;
        $event = this.eventData($event, item);
        this.invokeOnTouched();
        this.invokeOnChange(this.datavalue, $event || {});
        // updating the variable bound to default value as invokeOnChange is not updating the variable.
        if (this.datavalue !== this.prevDatavalue) {
            this.updateBoundVariable(this.datavalue);
        }
        if (this.$element.hasClass('full-screen')) {
            this.closeSearch();
        }
        this.invokeEventCallback('select', { $event, selectedValue: this.datavalue });
        this.invokeEventCallback('submit', { $event });
        this.updatePrevDatavalue(this.datavalue);
    }
    onPropertyChange(key, nv, ov) {
        if (key === 'tabindex') {
            return;
        }
        // Backward compatability from 10.3.1 to 10.3.2
        // 10.3.1 we had displaylabel as modal value
        // From 10.3.2 onwords we will be having displayexpresion as modal value
        if (this.type === 'autocomplete' && key === 'displaylabel' && this.displaylabel && (!this.displayexpression || (this.displayexpression && !this.datasource))) {
            this.displayexpression = this.displaylabel;
            this.displaylabel = undefined;
        }
        // when dataoptions are provided and there is no displaylabel given then displaylabel is set as the relatedfield
        if (key === 'displaylabel' && this.dataoptions && this.binddisplaylabel === null) {
            this.query = _.get(this._modelByValue, nv) || this._modelByValue;
        }
        super.onPropertyChange(key, nv, ov);
    }
};
SearchComponent.initializeProps = registerProps();
SearchComponent.ctorParameters = () => [
    { type: Injector },
    { type: undefined, decorators: [{ type: Attribute, args: ['datavalue.bind',] }] },
    { type: undefined, decorators: [{ type: Attribute, args: ['dataset.bind',] }] }
];
__decorate([
    ViewChild(TypeaheadDirective),
    __metadata("design:type", TypeaheadDirective)
], SearchComponent.prototype, "typeahead", void 0);
__decorate([
    ViewChild('ulElement'),
    __metadata("design:type", ElementRef)
], SearchComponent.prototype, "ulElement", void 0);
__decorate([
    ViewChildren('liElements'),
    __metadata("design:type", QueryList)
], SearchComponent.prototype, "liElements", void 0);
SearchComponent = SearchComponent_1 = __decorate([
    Component({
        selector: '[wmSearch]',
        template: "<ng-template #customItemTemplate let-model=\"item\" let-index=\"index\" let-query=\"query\" let-match=\"match\">\n    <!-- Default item template -->\n    <a *ngIf=\"!content\">\n        <img *ngIf=\"model.imgSrc\" [src]=\"model.imgSrc\" alt=\"Search\" [style.width]=\"imagewidth\">\n        <span [title]=\"model.label\" [innerHtml]=\"highlight(match, query) || model.label\"></span>\n    </a>\n    <!-- Custom partial template -->\n    <a *ngIf=\"content\" class=\"customTemplate\" partial-container-target>\n    </a>\n</ng-template>\n\n<ng-template #customListTemplate let-matches=\"matches\" let-itemTemplate=\"itemTemplate\" let-query=\"query\">\n    <ul #ulElement class=\"app-search dropdown-menu\" scrollable>\n        <li #liElements *ngFor=\"let match of matches\" [wmItemTemplate]=\"content\" [userComponentParams]=\"match.item\"  partialContainer [ngClass]=\"{active: typeaheadContainer && typeaheadContainer.isActive(match)}\"\n            (mouseenter)=\"typeaheadContainer.selectActive(match);\"\n            (click)=\"typeaheadContainer.selectMatch(match, $event); $typeaheadEvent = $event;\">\n            <!-- itemTemplate comes from the <input> -->\n            <ng-container *ngIf=\"!match.isHeader()\" [ngTemplateOutlet]=\"itemTemplate\"\n                          [ngTemplateOutletContext]=\"{item: match.item, index: i, match: match, query: query}\">\n            </ng-container>\n        </li>\n        <div class=\"status\" [hidden]=\"_loadingItems || !(datacompletemsg && dataProvider.isLastPage)\">\n            <span [textContent]=\"datacompletemsg\"></span>\n        </div>\n        <div class=\"status\" [hidden]=\"!_loadingItems\">\n            <i class=\"fa fa-circle-o-notch fa-spin\"></i>\n            <span [textContent]=\"loadingdatamsg\"></span>\n        </div>\n    </ul>\n</ng-template>\n\n<!--This template is for search inside mobile navbar.-->\n<ng-container *ngIf=\"navsearchbar; else searchTemplate\">\n    <input [title]=\"query || ''\" type=\"text\" class=\"app-textbox form-control list-of-objs app-search-input\" [placeholder]=\"placeholder || ''\"\n           focus-target\n           container=\"body\"\n           [disabled]=\"disabled\"\n           autocomplete=\"off\"\n           [(ngModel)]=\"query\"\n           [readonly]=\"readonly\"\n           [disabled]=\"disabled\"\n           [typeahead]=\"typeaheadDataSource\"\n           [typeaheadWaitMs]=\"debouncetime\"\n           [typeaheadItemTemplate]=\"customItemTemplate\"\n           [optionsListTemplate]=\"customListTemplate\"\n           (input)=\"onInputChange($event)\"\n           (keydown)=\"listenQuery = true\"\n           (keydown.enter)=\"$typeaheadEvent = $event;handleEnterEvent($event)\"\n           (keydown.arrowdown)=\"selectNext($event)\"\n           (blur)=\"invokeOnTouched($event)\"\n           (click)=\"handleFocus($event);\"\n           (focus)=\"_unsubscribeDv = false; listenQuery = true; handleFocus($event); invokeOnFocus($event);\"\n           (focusout)=\"onFocusOut()\"\n           [typeaheadMinLength]=\"minchars\"\n           [typeaheadOptionsLimit]=\"limit\"\n           (typeaheadLoading)=\"_loadingItems\"\n           (typeaheadOnSelect)=\"typeaheadOnSelect($event)\"\n           typeaheadOptionField=\"label\"\n           [typeaheadAsync]=\"true\"\n           [typeaheadScrollable]=\"true\"\n           [typeaheadOptionsInScrollableView]=\"optionslimitinscrollableview\"\n           [dropup]=\"dropup\"\n           [tabindex]=\"tabindex\"\n           [attr.name]=\"name\"/>\n    <i class=\"btn-close wi wi-cancel\" [hidden]=\"!showClosebtn\" (click)=\"clearSearch($event);\"></i>\n</ng-container>\n<!--This template is for both web and fullscreen mode in mobile.-->\n<ng-template #searchTemplate>\n    <span class=\"wi wi-arrow-left form-control-feedback back-btn\" aria-hidden=\"true\" (click)=\"closeSearch()\"></span>\n    <span class=\"sr-only\">Back button</span>\n    <input [title]=\"query || ''\" type=\"text\" class=\"app-textbox form-control list-of-objs app-search-input\" [placeholder]=\"placeholder || ''\"\n           focus-target\n           [container]=\"containerTarget || 'body'\"\n           [disabled]=\"disabled\"\n           autocomplete=\"off\"\n           [(ngModel)]=\"query\"\n           [readonly]=\"readonly\"\n           [typeahead]=\"typeaheadDataSource\"\n           [typeaheadWaitMs]=\"debouncetime\"\n           [typeaheadItemTemplate]=\"customItemTemplate\"\n           [optionsListTemplate]=\"customListTemplate\"\n           (input)=\"onInputChange($event)\"\n           (keydown)=\"listenQuery = true\"\n           (keydown.enter)=\"$typeaheadEvent = $event;handleEnterEvent($event)\"\n           (keydown.arrowdown)=\"selectNext($event)\"\n           (blur)=\"invokeOnTouched($event); onFocusOut()\"\n           (click)=\"handleFocus($event);\"\n           (focus)=\"_unsubscribeDv = false; listenQuery = true; handleFocus($event); invokeOnFocus($event)\"\n           [typeaheadMinLength]=\"minchars\"\n           [typeaheadOptionsLimit]=\"limit\"\n           (typeaheadLoading)=\"_loadingItems\"\n           (typeaheadOnSelect)=\"typeaheadOnSelect($event)\"\n           typeaheadOptionField=\"label\"\n           [typeaheadAsync]=\"true\"\n           [typeaheadScrollable]=\"true\"\n           [typeaheadOptionsInScrollableView]=\"optionslimitinscrollableview\"\n           [tabindex]=\"tabindex\"\n           [dropup]=\"dropup\"\n           [attr.name]=\"name\">\n    <input class=\"model-holder\" ng-model=\"proxyModel\" ng-required=\"required\" tabindex=\"-1\">\n    <span *ngIf=\"_loadingItems\" aria-hidden=\"true\" class=\"fa fa-circle-o-notch fa-spin form-control-feedback\"></span>\n    <span class=\"wi wi-close form-control-feedback clear-btn\" [hidden]=\"!showClosebtn\" (click)=\"clearSearch($event, true)\"></span>\n    <span class=\"sr-only\">Clear button</span>\n    <span *ngIf=\"showsearchicon\" class=\"input-group-addon\" aria-label=\"search icon\" [ngClass]=\"{'disabled': disabled}\">\n        <form>\n            <button title=\"Search\" [disabled]=\"disabled\" class=\"app-search-button wi wi-search\" type=\"submit\"\n                    (click)=\"onSearchSelect($event)\"></button>\n        </form>\n    </span>\n</ng-template>\n",
        providers: [
            provideAs(SearchComponent_1, NG_VALUE_ACCESSOR, true),
            provideAs(SearchComponent_1, NG_VALIDATORS, true),
            provideAsWidgetRef(SearchComponent_1)
        ]
    }),
    __param(1, Attribute('datavalue.bind')),
    __param(2, Attribute('dataset.bind')),
    __metadata("design:paramtypes", [Injector, Object, Object])
], SearchComponent);

let ScrollableDirective = class ScrollableDirective {
    constructor(inj, searchRef) {
        this.searchRef = searchRef;
        this.elementRef = inj.get(ElementRef);
    }
    ngAfterContentInit() {
        // add the scroll event listener on the ul element.
        this.elementRef.nativeElement.addEventListener('scroll', this.notifyParent.bind(this));
        this.searchRef.dropdownEl = $(this.elementRef.nativeElement);
        this.searchRef.onDropdownOpen();
    }
    ngAfterViewInit() {
        if (!this.searchRef.isMobileAutoComplete()) {
            // assigning width for the dropdown.
            const typeAheadInput = this.searchRef.$element.find('input').first();
            this.searchRef.dropdownEl.width(typeAheadInput.outerWidth());
        }
    }
    notifyParent(evt) {
        this.searchRef.onScroll(this.elementRef.nativeElement, evt);
    }
};
ScrollableDirective.ctorParameters = () => [
    { type: Injector },
    { type: SearchComponent }
];
ScrollableDirective = __decorate([
    Directive({
        selector: '[scrollable]'
    }),
    __metadata("design:paramtypes", [Injector, SearchComponent])
], ScrollableDirective);

const components = [
    SearchComponent,
    ScrollableDirective
];
let SearchModule = class SearchModule {
};
SearchModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            FormsModule,
            TypeaheadModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], SearchModule);

/**
 * Generated bundle index. Do not edit.
 */

export { SearchComponent, SearchModule, registerProps as ɵa, ScrollableDirective as ɵb };
//# sourceMappingURL=index.js.map
