import { __decorate, __metadata, __param } from 'tslib';
import { Injector, Attribute, HostBinding, ContentChildren, Component, QueryList, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { PROP_STRING, PROP_NUMBER, PROP_BOOLEAN, register, StylableComponent, styler, APPLY_STYLES_TYPE, RedrawableDirective, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { addClass, setCSSFromObj, setCSS, $appDigest, isMobileApp, noop, removeAttr, removeClass, appendNode, DynamicComponentRefProvider, StatePersistence } from '@wm/core';
import { SwipeAnimation } from '@swipey';

class TabsAnimator extends SwipeAnimation {
    constructor(tabs) {
        super();
        this.tabs = tabs;
        this._$el = $(this.tabs.getNativeElement()).find('>.tab-content');
        const childEls = this._$el.find('>[wmTabPane]');
        this._noOfTabs = childEls.length;
        const maxWidth = `${this._noOfTabs * 100}%`;
        addClass(this.tabs.getNativeElement(), 'has-transition');
        setCSSFromObj(this._$el[0], { maxWidth: maxWidth, width: maxWidth });
        const width = `${100 / this._noOfTabs}%`;
        for (const child of Array.from(childEls)) {
            setCSS(child, 'width', width);
        }
        this.init(this._$el);
    }
    bounds() {
        const activeTabIndex = this.tabs.getActiveTabIndex(), w = this._$el.find('>.tab-pane').first().width(), noOfTabs = this._$el.find('>.tab-pane:visible').length, centerVal = -1 * activeTabIndex * w;
        this.clearContentFocus();
        return {
            strict: false,
            lower: activeTabIndex === noOfTabs - 1 ? 0 : -w,
            center: centerVal,
            upper: activeTabIndex === 0 ? centerVal : w
        };
    }
    context() {
        return {
            'w': this._$el.width()
        };
    }
    animation() {
        return {
            'transform': 'translate3d(${{ ($D + $d)/w * 100 + \'%\'}}, 0, 0)',
            '-webkit-transform': 'translate3d(${{ ($D + $d)/w * 100 + \'%\'}}, 0, 0)'
        };
    }
    /* WMS-18031 | On Tab Switch | Remove the Focus on the Existing Tab Contents */
    clearContentFocus() {
        let activeTab = this._$el.find('>.tab-pane.active');
        if (activeTab.length && activeTab.find(document.activeElement).length) {
            document.activeElement.blur();
        }
    }
    transitionTabIntoView() {
        const activeTabIndex = this.tabs.getActiveTabIndex();
        setCSS(this._$el[0], 'transform', `translate3d(${-1 * activeTabIndex / this._noOfTabs * 100}%, 0, 0)`);
    }
    onUpper() {
        this.tabs.prev();
        $appDigest();
    }
    onLower() {
        this.tabs.next();
        $appDigest();
    }
    threshold() {
        return 5;
    }
}

const registerProps = () => {
    const props = new Map([
        ['class', PROP_STRING],
        ['defaultpaneindex', Object.assign({ value: 0 }, PROP_NUMBER)],
        ['justified', PROP_BOOLEAN],
        ['tabsposition', Object.assign({ value: 'top' }, PROP_STRING)],
        ['name', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['statehandler', Object.assign({ value: 'none' }, PROP_STRING)],
        ['transition', PROP_STRING]
    ]);
    if (isMobileApp()) {
        props.set('transition', Object.assign({ value: 'slide' }, PROP_STRING));
    }
    register('wm-tabs', props);
};

const registerProps$1 = () => {
    register('wm-tabpane', new Map([
        ['active', PROP_BOOLEAN],
        ['badgevalue', PROP_STRING],
        ['badgetype', Object.assign({ value: 'default' }, PROP_STRING)],
        ['class', PROP_STRING],
        ['content', PROP_STRING],
        ['disabled', PROP_BOOLEAN],
        ['dynamicPaneIndex', PROP_NUMBER],
        ['heading', PROP_STRING],
        ['isdefaulttab', PROP_STRING],
        ['isdynamic', PROP_BOOLEAN],
        ['name', PROP_STRING],
        ['paneicon', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['smoothscroll', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['tabindex', Object.assign({ value: 0 }, PROP_NUMBER)],
        ['title', Object.assign({ value: 'Tab Title' }, PROP_STRING)]
    ]));
};

var TabPaneComponent_1;
const DEFAULT_CLS = 'tab-pane';
const WIDGET_CONFIG = {
    widgetType: 'wm-tabpane',
    hostClass: DEFAULT_CLS
};
let TabPaneComponent = TabPaneComponent_1 = class TabPaneComponent extends StylableComponent {
    constructor(inj, tabsRef, heading, title) {
        super(inj, WIDGET_CONFIG);
        this.tabsRef = tabsRef;
        this.heading = heading;
        this.title = title;
        this._isFirstLoad = true;
        this.$lazyLoad = noop;
        this.isActive = false;
        this.disabled = false;
        // title property here serves the purpose of heading.
        // TODO: make it common for all the widget.
        removeAttr(this.nativeElement, 'title');
    }
    // parent tabs component will call this method for the order of callbacks to be proper
    // order of callbacks - deselect, select, change
    invokeOnSelectCallback($event) {
        this.invokeEventCallback('select', { $event });
    }
    select($event) {
        // When called programatically $event won't be available
        if (this.isActive || this.disabled) {
            return;
        }
        this.isActive = true;
        this.$lazyLoad();
        this.redrawChildren();
        this.notifyParent($event);
    }
    deselect() {
        if (this.isActive) {
            this.isActive = false;
            this.invokeEventCallback('deselect');
        }
    }
    remove() {
        if (this.isActive) {
            this === this.tabsRef.panes.last ? this.tabsRef.prev() : this.tabsRef.next();
        }
        const availablePanes = this.tabsRef.panes.toArray();
        availablePanes.splice(this.tabsRef.getPaneIndexByRef(this), 1);
        this.tabsRef.panes.reset([...availablePanes]);
        this.nativeElement.remove();
        this.tabsRef.$element.find(`.tab-header[data-paneid='${this.$element.attr('widget-id')}'] a`).remove();
    }
    redrawChildren() {
        setTimeout(() => {
            if (this.reDrawableComponents) {
                this.reDrawableComponents.forEach(c => c.redraw());
            }
        }, 100);
    }
    notifyParent(evt) {
        this.tabsRef.notifyChange(this, evt);
    }
    // select next valid tab
    handleSwipeLeft() {
        this.tabsRef.next();
    }
    // select prev valid tab
    handleSwipeRight() {
        this.tabsRef.prev();
    }
    // select event is called manually
    handleEvent(node, eventName, callback, locals) {
        if (eventName !== 'select') {
            super.handleEvent(this.nativeElement, eventName, callback, locals);
        }
    }
    onPropertyChange(key, nv, ov) {
        if (key === 'content') {
            if (this.isActive) {
                setTimeout(() => this.$lazyLoad(), 100);
            }
        }
        else if (key === 'show') {
            // overridding the base component show propertyChangeHandler
            this.nativeElement.hidden = !nv;
            // setting default tab pane on pageload whenever show property is bindable
            if (this._isFirstLoad) {
                this.tabsRef.selectDefaultPaneByIndex(this.tabsRef.defaultpaneindex);
            }
            this._isFirstLoad = false;
        }
        else {
            super.onPropertyChange(key, nv, ov);
        }
    }
    ngOnInit() {
        super.ngOnInit();
        this.title = this.title || this.heading;
    }
    ngAfterViewInit() {
        styler(this.nativeElement.querySelector('.tab-body'), this, APPLY_STYLES_TYPE.CONTAINER);
        super.ngAfterViewInit();
        if (this.isdynamic) {
            this.tabsRef.registerDynamicTab(this);
        }
    }
};
TabPaneComponent.initializeProps = registerProps$1();
TabPaneComponent.ctorParameters = () => [
    { type: Injector },
    { type: TabsComponent },
    { type: undefined, decorators: [{ type: Attribute, args: ['heading',] }] },
    { type: undefined, decorators: [{ type: Attribute, args: ['title',] }] }
];
__decorate([
    HostBinding('class.active'),
    __metadata("design:type", Object)
], TabPaneComponent.prototype, "isActive", void 0);
__decorate([
    HostBinding('class.disabled'),
    __metadata("design:type", Object)
], TabPaneComponent.prototype, "disabled", void 0);
__decorate([
    ContentChildren(RedrawableDirective, { descendants: true }),
    __metadata("design:type", Object)
], TabPaneComponent.prototype, "reDrawableComponents", void 0);
TabPaneComponent = TabPaneComponent_1 = __decorate([
    Component({
        selector: 'div[wmTabPane]',
        template: "<div class=\"tab-body\" partial-container-target [wmSmoothscroll]=\"smoothscroll\">\n    <ng-content></ng-content>\n</div>",
        providers: [
            provideAsWidgetRef(TabPaneComponent_1)
        ]
    }),
    __param(2, Attribute('heading')),
    __param(3, Attribute('title')),
    __metadata("design:paramtypes", [Injector,
        TabsComponent, Object, Object])
], TabPaneComponent);

var TabsComponent_1;
const DEFAULT_CLS$1 = 'app-tabs clearfix';
const WIDGET_CONFIG$1 = {
    widgetType: 'wm-tabs',
    hostClass: DEFAULT_CLS$1
};
let TabsComponent = TabsComponent_1 = class TabsComponent extends StylableComponent {
    constructor(inj, dynamicComponentProvider, _transition, _tabsPosition, statePersistence) {
        // handle to the promise resolver
        let resolveFn = noop;
        super(inj, WIDGET_CONFIG$1, new Promise(res => resolveFn = res));
        this.transition = _transition;
        this.tabsposition = _tabsPosition;
        this.statePersistence = statePersistence;
        this.dynamicComponentProvider = dynamicComponentProvider;
        this.dynamicTabs = [];
        this.dynamicPaneIndex = 0;
        this.promiseResolverFn = resolveFn;
        styler(this.nativeElement, this, APPLY_STYLES_TYPE.CONTAINER);
    }
    animateIn(element) {
        const tabHeader = $(element);
        // when the animation is not present toggle the active class.
        tabHeader.siblings('.active').removeClass('active');
        tabHeader.addClass('active');
        const ul = this.nativeElement.querySelector('ul.nav.nav-tabs');
        // move the tabheader into the viewport
        const $prevHeaderEle = tabHeader.prev();
        if ($prevHeaderEle.length) {
            ul.scrollLeft = $prevHeaderEle[0].offsetLeft;
        }
        else {
            ul.scrollLeft = 0;
        }
    }
    /**
     * This method is used to register the dynamic panes.
     * After all panes are initialzed, update the querylist manually based on index.
     * @param paneRef - refrence of the tabpane
     */
    registerDynamicTab(paneRef) {
        this.dynamicTabs.push(paneRef);
        const isLastPane = this.dynamicTabs.length === this.dynamicPaneIndex;
        if (isLastPane) {
            for (let i = 0; i < this.dynamicTabs.length; i++) {
                const newPaneRef = _.find(this.dynamicTabs, pane => pane.dynamicPaneIndex === i);
                const isPaneAlreadyExist = _.find(this.panes.toArray(), newPaneRef);
                if (!isPaneAlreadyExist) {
                    this.panes.reset([...this.panes.toArray(), newPaneRef]);
                    if (newPaneRef.active) {
                        setTimeout(() => {
                            newPaneRef.select();
                        }, 20);
                    }
                }
            }
        }
    }
    /**
     * This method is to add the tabpane dynamically
     * @param tabpanes - list of tabpanes
     */
    addPane(tabpanes) {
        if (!_.isArray(tabpanes)) {
            tabpanes = [tabpanes];
        }
        const paneNamesList = [];
        _.forEach(tabpanes, (pane, index) => {
            const isPaneAlreadyCreated = _.find(this.panes.toArray(), { name: pane.name });
            const isPaneNameExist = _.indexOf(paneNamesList, pane.name);
            // If user tries to add tabpane with the same name which is already exists then do not create the pane
            if (isPaneAlreadyCreated || isPaneNameExist > 0) {
                console.warn(`The tab pane with name ${pane.name} already exists`);
                return;
            }
            let paramMarkup = '';
            let propsTmpl = '';
            this.dynamicPaneIndex++;
            const name = pane.name ? pane.name : `tabpane${this.panes.toArray().length + (index + 1)}`;
            paneNamesList.push(name);
            const partialParams = _.get(pane, 'params');
            _.forEach(pane, (value, key) => {
                if (key !== 'params') {
                    propsTmpl = `${propsTmpl} ${key}="${value}"`;
                }
            });
            _.forEach(partialParams, (value, key) => {
                paramMarkup = `${paramMarkup} <wm-param name="${key}" value="${value}"></wm-param>`;
            });
            const markup = `<wm-tabpane dynamicPaneIndex="${this.dynamicPaneIndex - 1}" isdynamic="true" name="${name}" ${propsTmpl}>
                            ${paramMarkup}
                        </wm-tabpane>`;
            if (!this._dynamicContext) {
                this._dynamicContext = Object.create(this.viewParent);
                this._dynamicContext[this.getAttr('wmTab')] = this;
            }
            this.dynamicComponentProvider.addComponent(this.getNativeElement().querySelector('.tab-content'), markup, this._dynamicContext, { inj: this.inj });
        });
        return paneNamesList;
    }
    /**
     * This method is to remove the tabpane
     * @param paneIndex - index of the pane
     */
    removePane(paneName) {
        const paneRef = this.getPaneRefByName(paneName);
        if (paneRef) {
            paneRef.remove();
        }
    }
    /**
     * TabPane children components invoke this method to communicate with the parent
     * if the evt argument is defined on-change callback will be invoked.
     */
    notifyChange(paneRef, evt) {
        if (!this.isSelectableTab(paneRef)) {
            return;
        }
        let headerElement;
        // invoke deselect event callback on the preset active tab
        if (this.activeTab) {
            this.activeTab.deselect();
        }
        // invoke select callback on the selected tab
        paneRef.invokeOnSelectCallback(evt);
        this.activeTab = paneRef.getWidget();
        const newPaneIndex = this.getPaneIndexByRef(paneRef);
        const mode = this.statePersistence.computeMode(this.statehandler);
        if (!this.isPageLoadCall && mode && mode.toLowerCase() !== 'none') {
            this.statePersistence.setWidgetState(this, this.activeTab.name);
        }
        else {
            this.defaultpaneindex = this.getPaneIndexByRef(paneRef);
            this.isPageLoadCall = false;
        }
        // invoke change callback if the evt is present, select a tab programmatically will not have the event
        if (evt) {
            this.invokeEventCallback('change', {
                $event: evt,
                newPaneIndex: newPaneIndex,
                oldPaneIndex: this._oldPaneIndex || 0
            });
        }
        this._oldPaneIndex = newPaneIndex;
        if (evt) {
            headerElement = $(evt.target).closest('li.tab-header');
        }
        else {
            headerElement = this.nativeElement.querySelector(`li[data-paneid=${paneRef.widgetId}]`);
        }
        this.animateIn(headerElement);
        // this.setTabsLeftPosition(this.getPaneIndexByRef(this.activeTab), this.panes.length);
        if (this.canSlide()) {
            if (!this.tabsAnimator) {
                this.tabsAnimator = new TabsAnimator(this);
                this.tabsAnimator.setGesturesEnabled(this.canSlide());
            }
            this.tabsAnimator.transitionTabIntoView();
        }
    }
    goToTab(tabIndex) {
        if (this.isValidPaneIndex(tabIndex - 1)) {
            const tab = this.getPaneRefByIndex(tabIndex - 1);
            tab.select();
        }
    }
    getPaneIndexByRef(paneRef) {
        return this.panes.toArray().indexOf(paneRef);
    }
    // Returns the active tab index from tabs.
    getActiveTabIndex() {
        return _.findIndex(this.panes.toArray(), { isActive: true });
    }
    isValidPaneIndex(index) {
        return (index >= 0 && index < this.panes.length);
    }
    getPaneRefByIndex(index) {
        return this.panes.toArray()[index];
    }
    getPaneRefByName(name) {
        return _.find(this.panes.toArray(), { name: name });
    }
    // returns false if the pane is hidden or disabled
    isSelectableTab(paneRef) {
        return paneRef.show && !paneRef.disabled;
    }
    canSlide() {
        return this.transition === 'slide' && !this.vertical;
    }
    getSelectableTabAfterIndex(index) {
        for (let i = index + 1; i < this.panes.length; i++) {
            const pane = this.getPaneRefByIndex(i);
            if (this.isSelectableTab(pane)) {
                return pane;
            }
        }
    }
    getSelectableTabBeforeIndex(index) {
        for (let i = index - 1; i >= 0; i--) {
            const pane = this.getPaneRefByIndex(i);
            if (this.isSelectableTab(pane)) {
                return pane;
            }
        }
    }
    // select next tab relative to the current active tab
    next() {
        const pane = this.getSelectableTabAfterIndex(this.getActiveTabIndex());
        if (pane) {
            pane.select();
        }
    }
    // select prev tab relative to the current active tab
    prev() {
        const pane = this.getSelectableTabBeforeIndex(this.getActiveTabIndex());
        if (pane) {
            pane.select();
        }
    }
    /**
     * this method will be invoked during the initialization of the component and on defaultpaneindex property change,
     * if the provided defaultpaneindex is not valid, find the first pane index which can be shown and select it
     */
    selectDefaultPaneByIndex(index) {
        if (!this.isValidPaneIndex(index)) {
            return;
        }
        let paneRef = this.getPaneRefByIndex(index);
        if (!this.isSelectableTab(paneRef)) {
            paneRef = this.getSelectableTabAfterIndex(0);
        }
        if (paneRef) {
            paneRef.select();
        }
    }
    // update the postion of tab header
    setTabsPosition() {
        const ul = this.nativeElement.children[0];
        this.vertical = (this.tabsposition === 'left' || this.tabsposition === 'right');
        removeClass(this.nativeElement, 'inverted');
        if (this.tabsposition === 'bottom' || this.tabsposition === 'right') {
            appendNode(ul, this.nativeElement);
            addClass(this.nativeElement, 'inverted');
        }
    }
    onPropertyChange(key, nv, ov) {
        if (key === 'defaultpaneindex') {
            this.defaultpaneindex = nv;
        }
        else if (key === 'statehandler') {
            this.isPageLoadCall = true;
            const widgetState = this.statePersistence.getWidgetState(this);
            if (nv !== 'none' && widgetState) {
                const paneToSelect = this.panes.filter(function (pane) {
                    return widgetState === pane.name;
                });
                if (!paneToSelect.length) {
                    console.warn('Tab pane name ' + widgetState + ' in State is incorrect. Falling back to the default pane');
                    setTimeout(() => this.selectDefaultPaneByIndex(this.defaultpaneindex || 0), 20);
                }
                else {
                    const index = this.getPaneIndexByRef(paneToSelect[0]);
                    setTimeout(() => this.selectDefaultPaneByIndex(index), 20);
                }
            }
            else {
                setTimeout(() => this.selectDefaultPaneByIndex(this.defaultpaneindex || 0), 20);
            }
        }
        else {
            super.onPropertyChange(key, nv, ov);
        }
    }
    registerTabsScroll() {
        setTimeout(() => {
            const $ul = this.$element.find('> ul');
            let $liPosition;
            const $li = $ul.children();
            $liPosition = $li.last().position();
            if ($liPosition && ($liPosition.left > $ul.width())) {
                $ul.on('mousewheel', (e) => {
                    const left = $ul[0].scrollLeft, _delta = -1 * e.originalEvent.wheelDelta;
                    e.stopPropagation();
                    e.preventDefault();
                    $ul.animate({ scrollLeft: left + _delta }, { 'duration': 10 });
                });
            }
        });
    }
    ngAfterContentInit() {
        this.promiseResolverFn();
        super.ngAfterContentInit();
        this.setTabsPosition();
    }
    ngAfterViewInit() {
        super.ngAfterViewInit();
        this.registerTabsScroll();
    }
};
TabsComponent.initializeProps = registerProps();
TabsComponent.ctorParameters = () => [
    { type: Injector },
    { type: DynamicComponentRefProvider },
    { type: String, decorators: [{ type: Attribute, args: ['transition',] }] },
    { type: String, decorators: [{ type: Attribute, args: ['tabsposition',] }] },
    { type: StatePersistence }
];
__decorate([
    ContentChildren(TabPaneComponent),
    __metadata("design:type", QueryList)
], TabsComponent.prototype, "panes", void 0);
TabsComponent = TabsComponent_1 = __decorate([
    Component({
        selector: 'div[wmTabs]',
        template: "<ul class=\"nav nav-tabs\" [ngClass]=\"{'nav-stacked': vertical, 'nav-justified': justified}\" role=\"tablist\">\n    <li class=\"tab-header\" *ngFor=\"let pane of panes;\" [attr.data-paneid]=\"pane.widgetId\" [ngClass]=\"{'active': pane.isActive, 'disabled': pane.disabled}\"\n        [hidden]=\"!pane.show\" (click)=\"pane.select($event)\" role=\"tab\">\n        <a href=\"javascript:void(0);\" role=\"button\" [attr.aria-label]=\"pane.title\" [attr.title]=\"pane.title\" [tabindex]=\"pane.tabindex\">\n            <div class=\"tab-heading\">\n                <i [ngClass]=\"['app-icon', pane.paneicon]\" *ngIf=\"pane.paneicon\"></i>\n                <span [textContent]=\"pane.title\"></span>\n                <span *ngIf=\"pane.badgevalue\" class=\"label label-{{pane.badgetype}}\" [textContent]=\"pane.badgevalue\"></span>\n            </div>\n        </a>\n    </li>\n</ul>\n<div class=\"tab-content\" [ngClass]=\"{'tab-stacked': vertical, 'tab-justified': justified}\">\n    <ng-content select=\"div[wmTabPane]\"></ng-content>\n</div>\n",
        providers: [
            provideAsWidgetRef(TabsComponent_1)
        ]
    }),
    __param(2, Attribute('transition')),
    __param(3, Attribute('tabsposition')),
    __metadata("design:paramtypes", [Injector,
        DynamicComponentRefProvider, String, String, StatePersistence])
], TabsComponent);

const components = [
    TabPaneComponent,
    TabsComponent
];
let TabsModule = class TabsModule {
};
TabsModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], TabsModule);

/**
 * Generated bundle index. Do not edit.
 */

export { TabsModule, TabPaneComponent as ɵa, TabsComponent as ɵb, registerProps as ɵc, registerProps$1 as ɵd };
//# sourceMappingURL=index.js.map
