import { __decorate, __metadata, __param } from 'tslib';
import { ElementRef, Inject, HostBinding, Directive, Injector, Input, Component, NgModule } from '@angular/core';
import { BsModalService, ModalModule } from 'ngx-bootstrap/modal';
import { addClass, setCSS, setAttr, findRootContainer, AbstractDialogService, toBoolean } from '@wm/core';
import { DialogRef, register, PROP_STRING, PROP_BOOLEAN, BaseComponent, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { Router, NavigationStart } from '@angular/router';
import { CommonModule } from '@angular/common';

const DEFAULT_CLS = 'app-dialog-body modal-body';
let DialogBodyDirective = class DialogBodyDirective {
    constructor(elRef, dialogRef, bsModal) {
        this.dialogRef = dialogRef;
        addClass(elRef.nativeElement, DEFAULT_CLS);
        const subscription = bsModal.onShown.subscribe(() => {
            const dialogRoot = $(elRef.nativeElement).closest('.app-dialog')[0];
            const width = this.dialogRef.width;
            const height = this.dialogRef.height;
            if (height) {
                setTimeout(e => {
                    setCSS($(elRef.nativeElement)[0], 'height', height);
                });
            }
            if (dialogRoot) {
                if (width) {
                    setCSS(dialogRoot, 'width', width);
                }
                setAttr(dialogRoot, 'tabindex', this.dialogRef.tabindex);
                setAttr(dialogRoot, 'name', this.dialogRef.name);
            }
            subscription.unsubscribe();
        });
    }
};
DialogBodyDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: undefined, decorators: [{ type: Inject, args: [DialogRef,] }] },
    { type: BsModalService }
];
__decorate([
    HostBinding('style.height'),
    __metadata("design:type", Object)
], DialogBodyDirective.prototype, "height", void 0);
DialogBodyDirective = __decorate([
    Directive({
        selector: 'div[wmDialogBody]',
    }),
    __param(1, Inject(DialogRef)),
    __metadata("design:paramtypes", [ElementRef, Object, BsModalService])
], DialogBodyDirective);

const registerProps = () => {
    register('wm-dialogfooter', new Map([
        ['class', PROP_STRING],
        ['name', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var DialogFooterDirective_1;
const WIDGET_INFO = {
    widgetType: 'wm-dialogfooter',
    hostClass: 'app-dialog-footer modal-footer'
};
let DialogFooterDirective = DialogFooterDirective_1 = class DialogFooterDirective extends BaseComponent {
    constructor(inj) {
        super(inj, WIDGET_INFO);
    }
};
DialogFooterDirective.initializeProps = registerProps();
DialogFooterDirective.ctorParameters = () => [
    { type: Injector }
];
DialogFooterDirective = DialogFooterDirective_1 = __decorate([
    Directive({
        selector: 'div[wmDialogFooter]',
        providers: [
            provideAsWidgetRef(DialogFooterDirective_1)
        ]
    }),
    __metadata("design:paramtypes", [Injector])
], DialogFooterDirective);

let eventsRegistered = false;
const invokeOpenedCallback = (ref) => {
    if (ref) {
        setTimeout(() => {
            const root = findRootContainer(ref.$element);
            // if page styles have to be applied to dialog then dialog has to be child of page element.
            if (root) {
                $('body > modal-container > div').wrap('<' + root + '/>');
            }
            ref.invokeEventCallback('opened', { $event: { type: 'opened' } });
        });
    }
};
const ɵ0 = invokeOpenedCallback;
const invokeClosedCallback = (ref) => {
    if (ref) {
        ref.invokeEventCallback('close');
        ref.dialogRef = undefined;
    }
};
const ɵ1 = invokeClosedCallback;
class BaseDialog extends BaseComponent {
    constructor(inj, widgetConfig, modalOptions) {
        super(inj, widgetConfig);
        this.modalOptions = modalOptions;
        this.dialogService = inj.get(AbstractDialogService);
        this.bsModal = inj.get(BsModalService);
        const router = inj.get(Router);
        const subscriptions = [
            this.bsModal.onShown.subscribe(() => {
                const ref = this.dialogService.getLastOpenedDialog();
                if (ref === this) {
                    // Always get the reference of last pushed dialog in the array for calling onOpen callback
                    invokeOpenedCallback(ref);
                }
            }),
            this.bsModal.onHidden.subscribe((closeReason) => {
                const ref = closeReason === 'esc' || closeReason === 'backdrop-click' ? this.dialogService.getLastOpenedDialog() : this.dialogService.getDialogRefFromClosedDialogs();
                if (ref === this) {
                    // remove the dialog reference from opened dialogs and closed dialogs
                    this.dialogService.removeFromOpenedDialogs(ref);
                    this.dialogService.removeFromClosedDialogs(ref);
                    invokeClosedCallback(ref);
                    if (ref.closeCallBackFn) {
                        ref.closeCallBackFn();
                    }
                }
            }),
            router.events.subscribe(e => {
                if (e instanceof NavigationStart) {
                    this.close();
                }
            })
        ];
        this.registerDestroyListener(() => {
            this.dialogService.removeFromOpenedDialogs(this);
            this.dialogService.removeFromClosedDialogs(this);
            subscriptions.forEach(s => s.unsubscribe());
        });
    }
    /**
     * Opens the dialog
     * Subscribe to the onShown event emitter of bsModal and trigger on-opened event callback
     */
    open(initState) {
        // remove the popovers in the page to avoid the overlap with dialog
        // closePopover(this.$element); Commenting this line because it is causing regression(if we have dialog inside popover as partail content, then the dialog close is not working because on closing the popover the partial get destroyed.)
        // do not open the dialog again if it is already opened
        const duplicateDialogCheck = (openedDialog) => {
            return openedDialog === this;
        };
        if (this.dialogService.getOpenedDialogs().some(duplicateDialogCheck)) {
            return;
        }
        this.dialogService.addToOpenedDialogs(this);
        // extend the context with the initState
        Object.assign(this.context, initState);
        this.dialogRef = this.bsModal.show(this.getTemplateRef(), this.modalOptions);
    }
    /**
     * closes the dialog
     * invokes the on-close event callback
     */
    close() {
        // remove the popovers in the page to avoid the overlap with dialog
        // closePopover(this.$element); Commenting this line because it is causing regression(if we have dialog inside popover as partail content, then the dialog close is not working because on closing the popover the partial get destroyed.)
        if (this.dialogRef) {
            this.dialogService.addToClosedDialogs(this);
            this.dialogRef.hide();
        }
    }
    /**
     * Register the dialog with the dialog service for programmatic access
     */
    register(scope) {
        // add scope along with name in the dialogRefsCollection Map while registering dialog
        // So that 2 dialogs having same name on different pages won't be overridden.
        this.dialogService.register(this.name, this, scope);
    }
    /**
     * De Register the dialog with the dialog service after dialog destruction
     */
    deRegister(scope) {
        this.dialogService.deRegister(this.name, scope);
    }
    onPropertyChange(key, nv, ov) {
        // ignore the class attribute.
        // Prevent the framework from setting the class on the host element.
        if (key === 'class' || key === 'name' || key === 'tabindex') {
            return;
        }
        else if (key === 'animation') {
            this.modalOptions.class = this.modalOptions.class.replace(' animated ' + ov, '');
            if (nv) {
                this.modalOptions.class = this.modalOptions.class + ' animated ' + nv;
            }
        }
        super.onPropertyChange(key, nv, ov);
    }
    ngOnDestroy() {
        this.close();
        this.deRegister(this.viewParent);
        super.ngOnDestroy();
    }
}

const DEFAULT_CLS$1 = 'app-dialog-header modal-header';
const DEFAULT_ICON_DIMENSIONS = '21px';
let DialogHeaderComponent = class DialogHeaderComponent {
    constructor(elRef, dialogRef) {
        this.dialogRef = dialogRef;
        this.iconwidth = DEFAULT_ICON_DIMENSIONS;
        this.iconheight = DEFAULT_ICON_DIMENSIONS;
        this.closable = true;
        addClass(elRef.nativeElement, DEFAULT_CLS$1);
    }
    get isClosable() {
        return toBoolean(this.closable);
    }
    closeDialog() {
        this.dialogRef.close();
    }
};
DialogHeaderComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: BaseDialog, decorators: [{ type: Inject, args: [DialogRef,] }] }
];
__decorate([
    Input(),
    __metadata("design:type", Object)
], DialogHeaderComponent.prototype, "iconwidth", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], DialogHeaderComponent.prototype, "iconheight", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], DialogHeaderComponent.prototype, "iconmargin", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], DialogHeaderComponent.prototype, "iconclass", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], DialogHeaderComponent.prototype, "iconurl", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], DialogHeaderComponent.prototype, "closable", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], DialogHeaderComponent.prototype, "heading", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], DialogHeaderComponent.prototype, "subheading", void 0);
DialogHeaderComponent = __decorate([
    Component({
        selector: 'div[wmDialogHeader]',
        template: "<button *ngIf=\"isClosable\" aria-label=\"Close\" class=\"app-dialog-close close\" (click)=\"closeDialog();\" title=\"Close\">\n    <span aria-hidden=\"true\">&times;</span>\n</button>\n<h4 class=\"app-dialog-title modal-title\">\n    <i [ngClass]=\"iconclass\" [ngStyle]=\"{width: iconwidth, height: iconheight, margin: iconmargin}\"  *ngIf=\"iconclass && !iconurl\"></i>\n    <img data-identifier=\"img\" [src]=\"iconurl | image\" *ngIf=\"iconurl\" [ngStyle]=\"{width:iconwidth, height:iconheight, margin:iconmargin}\"/>\n    <span class=\"dialog-heading\" [textContent]=\"heading\"></span>\n    <span class=\"dialog-sub-heading\" *ngIf=\"subheading\" [title]=\"subheading\" [textContent]=\"subheading\"></span>\n</h4>"
    }),
    __param(1, Inject(DialogRef)),
    __metadata("design:paramtypes", [ElementRef, BaseDialog])
], DialogHeaderComponent);

const components = [
    DialogBodyDirective,
    DialogFooterDirective,
    DialogHeaderComponent
];
let DialogModule = class DialogModule {
};
DialogModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            ModalModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], DialogModule);

/**
 * Generated bundle index. Do not edit.
 */

export { BaseDialog, DialogBodyDirective, DialogFooterDirective, DialogHeaderComponent, DialogModule, ɵ0, ɵ1, registerProps as ɵa };
//# sourceMappingURL=index.js.map
