import { __decorate, __metadata, __param } from 'tslib';
import { Injector, ViewChild, Directive, ChangeDetectorRef, Inject, Component, NgZone, NgModule } from '@angular/core';
import { getLocaleDayPeriods, FormStyle, TranslationWidth, CommonModule } from '@angular/common';
import { NG_VALUE_ACCESSOR, NG_VALIDATORS, FormsModule } from '@angular/forms';
import { BsDatepickerConfig, BsDatepickerDirective, BsDatepickerModule } from 'ngx-bootstrap/datepicker';
import { BsDropdownDirective, BsDropdownModule } from 'ngx-bootstrap/dropdown';
import { TimepickerConfig, TimepickerComponent, TimepickerModule } from 'ngx-bootstrap/timepicker';
import { ToDatePipe, getContainerTargetClass, PROP_BOOLEAN, PROP_STRING, PROP_ANY, PROP_NUMBER, register, registerFormWidget, styler, provideAs, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { EVENT_MANAGER_PLUGINS } from '@angular/platform-browser';
import { AbstractI18nService, isMobile, getFormattedDate, getDateObj, isString, setAttr, getNativeDateObject, hasCordova, FormWidgetType, getDisplayDateTimeFormat, adjustContainerPosition, adjustContainerRightEdges, addEventListenerOnElement, AppDefaults, addClass, $appDigest } from '@wm/core';
import { BaseFormCustomComponent } from '@wm/components/input';

const CURRENT_DATE = 'CURRENT_DATE';
const months = ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'];
const DATEPICKER_DROPDOWN_OPTIONS = {
    BUTTON: 'button',
    DEFAULT: 'default'
};
const DATAENTRYMODE_DROPDOWN_OPTIONS = {
    PICKER: 'picker',
    DEFAULT: 'default'
};
// Providing meridians to the timepicker baesd on selected locale
function getTimepickerConfig(i18nService) {
    return Object.assign(new TimepickerConfig(), {
        meridians: getLocaleDayPeriods(i18nService.getSelectedLocale(), FormStyle.Format, TranslationWidth.Abbreviated)
    });
}
let BaseDateTimeComponent = class BaseDateTimeComponent extends BaseFormCustomComponent {
    constructor(inj, WIDGET_CONFIG) {
        super(inj, WIDGET_CONFIG);
        this.isReadOnly = false;
        /**
         * This is an internal property used to map the containerClass, showWeekNumbers etc., to the bsDatepicker
         */
        this._dateOptions = new BsDatepickerConfig();
        this.datePipe = this.inj.get(ToDatePipe);
        this.i18nService = this.inj.get(AbstractI18nService);
        this.selectedLocale = this.i18nService.getSelectedLocale();
        this.meridians = getLocaleDayPeriods(this.selectedLocale, FormStyle.Format, TranslationWidth.Abbreviated);
        this.loadNativeDateInput = isMobile();
    }
    /**
     * returns true if the input value is default (i.e open date picker on input click)
     * @param1 dropdownvalue, user selected value (by default datepicker opens on input click)
     * **/
    isDropDownDisplayEnabledOnInput(dropdownvalue) {
        return dropdownvalue === DATEPICKER_DROPDOWN_OPTIONS.DEFAULT;
    }
    /**
     * returns true if the input value is default (i.e Data entry can be done either by selecting from the Date/DateTime/Time Picker or by entering  manually using the keyboard. )
     * @param1 dropdownvalue, user selected value
     * **/
    isDataEntryModeEnabledOnInput(dropdownvalue) {
        return dropdownvalue === DATAENTRYMODE_DROPDOWN_OPTIONS.DEFAULT;
    }
    /**
     * This method is used to show validation message depending on the isNativePicker flag.
     */
    showValidation($event, displayValue, isNativePicker, msg) {
        if (isNativePicker) {
            console.warn('min max date validation failed ', msg);
            return $($event.target).val(displayValue);
        }
    }
    resetDisplayInput() {
        $(this.nativeElement).find('.display-input').val('');
    }
    validate(c) {
        if (this.invalidDateTimeFormat) {
            return {
                invalidDateTimeFormat: {
                    valid: false
                }
            };
        }
        if (!_.isUndefined(this.dateNotInRange) && this.dateNotInRange) {
            return {
                dateNotInRange: {
                    valid: false
                },
            };
        }
        if (!_.isUndefined(this.timeNotInRange) && this.timeNotInRange) {
            return {
                timeNotInRange: {
                    valid: false
                },
            };
        }
        /* WMS-18269 | Extending the existing validation for 'required' */
        if (this["show"] && this["required"]) {
            return !!c.value ? null : { required: true };
        }
        this.validateType = '';
        return null;
    }
    /**
     * This method is used to validate date pattern and time pattern
     * If user selects one pattern in design time and if he tries to enter the date in another pattern then the device is throwing an error
     */
    formatValidation(newVal, inputVal, isNativePicker) {
        const pattern = this.datepattern || this.timepattern;
        const formattedDate = getFormattedDate(this.datePipe, newVal, pattern);
        inputVal = inputVal.trim();
        if (inputVal) {
            if (pattern === 'timestamp') {
                if (!_.isNaN(inputVal) && _.parseInt(inputVal) !== formattedDate) {
                    this.invalidDateTimeFormat = true;
                    this.validateType = 'incorrectformat';
                    this.invokeOnChange(this.datavalue, undefined, false);
                    return false;
                }
            }
            else {
                if (isNativePicker) {
                    // format the date value only when inputVal is obtained from $event.target.value, as the format doesnt match.
                    inputVal = getFormattedDate(this.datePipe, inputVal, pattern);
                }
                if (inputVal !== formattedDate) {
                    this.invalidDateTimeFormat = true;
                    this.validateType = 'incorrectformat';
                    this.invokeOnChange(this.datavalue, undefined, false);
                    return false;
                }
            }
        }
        return true;
    }
    /**
     * This method is used to validate min date, max date, exclude dates and exclude days
     * In mobile if invalid dates are entered, device is showing an alert.
     * In web if invalid dates are entered, device is showing validation message.
     */
    minDateMaxDateValidationOnInput(newVal, $event, displayValue, isNativePicker) {
        if (newVal) {
            newVal = moment(newVal).startOf('day').toDate();
            const minDate = moment(getDateObj(this.mindate)).startOf('day').toDate();
            const maxDate = moment(getDateObj(this.maxdate)).startOf('day').toDate();
            if (this.mindate && newVal < minDate) {
                const msg = `${this.appLocale.LABEL_MINDATE_VALIDATION_MESSAGE} ${this.mindate}.`;
                this.dateNotInRange = true;
                this.validateType = 'mindate';
                this.invokeOnChange(this.datavalue, undefined, false);
                return this.showValidation($event, displayValue, isNativePicker, msg);
            }
            if (this.maxdate && newVal > maxDate) {
                const msg = `${this.appLocale.LABEL_MAXDATE_VALIDATION_MESSAGE} ${this.maxdate}.`;
                this.dateNotInRange = true;
                this.validateType = 'maxdate';
                this.invokeOnChange(this.datavalue, undefined, false);
                return this.showValidation($event, displayValue, isNativePicker, msg);
            }
            if (this.excludedates) {
                let excludeDatesArray;
                if (isString(this.excludedates)) {
                    excludeDatesArray = _.split(this.excludedates, ',');
                }
                else {
                    excludeDatesArray = this.excludedates;
                }
                excludeDatesArray = excludeDatesArray.map(d => getFormattedDate(this.datePipe, d, this.datepattern));
                if (excludeDatesArray.indexOf(getFormattedDate(this.datePipe, newVal, this.datepattern)) > -1) {
                    this.dateNotInRange = true;
                    this.validateType = 'excludedates';
                    this.invokeOnChange(this.datavalue, undefined, false);
                    return;
                }
            }
            if (this.excludedays) {
                const excludeDaysArray = _.split(this.excludedays, ',');
                if (excludeDaysArray.indexOf(newVal.getDay().toString()) > -1) {
                    this.dateNotInRange = true;
                    this.validateType = 'excludedays';
                    this.invokeOnChange(this.datavalue, undefined, false);
                    return;
                }
            }
        }
        if (!isNativePicker) {
            this.dateNotInRange = false;
            this.invokeOnChange(this.datavalue, undefined, false);
        }
    }
    /**
     * This method is used to highlight the current date
     */
    hightlightToday() {
        const toDay = new Date().getDate().toString();
        _.filter($(`span:contains(${toDay})`).not('.is-other-month'), (obj) => {
            if ($(obj).text() === toDay) {
                $(obj).addClass('current-date text-info');
            }
        });
    }
    /**
     * This method is used to find the new date is from another year or not
     * @param newDate - newly selected date value
     */
    isOtheryear(newDate) {
        return (newDate.getMonth() === 0 && this.activeDate.getMonth() === 11) || (newDate.getMonth() === 11 && this.activeDate.getMonth() === 0);
    }
    /**
     * This method is used to set focus for active day
     * @param newDate - newly selected date value
     * @param isMouseEvent - boolean value represents the event is mouse event/ keyboard event
     */
    setActiveDateFocus(newDate, isMouseEvent) {
        const activeMonth = this.activeDate.getMonth();
        // check for keyboard event
        if (!isMouseEvent) {
            if (newDate.getMonth() < activeMonth) {
                this.isOtheryear(newDate) ? this.goToOtherMonthOryear('next', 'days') : this.goToOtherMonthOryear('previous', 'days');
            }
            else if (newDate.getMonth() > activeMonth) {
                this.isOtheryear(newDate) ? this.goToOtherMonthOryear('previous', 'days') : this.goToOtherMonthOryear('next', 'days');
            }
        }
        setTimeout(() => {
            if (newDate.getMonth() === new Date().getMonth() && newDate.getFullYear() === new Date().getFullYear()) {
                this.hightlightToday();
            }
            const newDay = newDate.getDate().toString();
            _.filter($(`span:contains(${newDay})`).not('.is-other-month'), (obj) => {
                if ($(obj).text() === newDay) {
                    $(obj).focus();
                    this.activeDate = newDate;
                }
            });
        });
    }
    /**
     * This method is used to load other month days or other month or other year
     * @param btnClass - class(previous/next) of the button which we have to click
     * @param timePeriod - String value decides to load other month days or other month or other year
     */
    goToOtherMonthOryear(btnClass, timePeriod) {
        const $node = $(`.bs-datepicker-head .${btnClass}`);
        if ($node.attr('disabled')) {
            return;
        }
        $node.trigger('click');
        if (timePeriod === 'days') {
            this.loadDays();
        }
        else if (timePeriod === 'month') {
            this.loadMonths();
        }
        else if (timePeriod === 'year') {
            this.loadYears();
        }
    }
    /**
    * This method sets the mouse events to Datepicker popup. These events are required when we navigate date picker through mouse.
     */
    addDatepickerMouseEvents() {
        const datePikcerHead = $(`.bs-datepicker-head`);
        datePikcerHead.find(`.previous`).click((event) => {
            // check for original mouse click
            if (event.originalEvent) {
                this.setFocusForDate(-1);
            }
        });
        datePikcerHead.find(`.next`).click((event) => {
            // check for original mouse click
            if (event.originalEvent) {
                this.setFocusForDate(1);
            }
        });
        datePikcerHead.find(`.current`).click((event) => {
            // check for original mouse click
            if (event.originalEvent) {
                this.setFocusForCurrentMonthOryear();
            }
        });
        $('.bs-datepicker-body, .bs-datepicker-action-buttons').click((event) => {
            event.stopPropagation();
            // check for original mouse click
            if (event.originalEvent) {
                this.setFocusForMonthOrDay();
            }
        });
    }
    /**
     * This method sets focus for months/days depending on the loaded datepicker table
     */
    setFocusForMonthOrDay() {
        const activeMonthOrYear = $(`.bs-datepicker-head .current`).first().text();
        const datePickerBody = $('.bs-datepicker-body');
        if (datePickerBody.find('table.months').length > 0) {
            if (_.parseInt(activeMonthOrYear) !== this.activeDate.getFullYear()) {
                this.loadMonths();
            }
            const newDate = new Date(_.parseInt(activeMonthOrYear), this.activeDate.getMonth(), this.activeDate.getDate());
            this.setActiveMonthFocus(newDate, true);
        }
        else if (datePickerBody.find('table.days').length > 0) {
            const newMonth = months.indexOf(activeMonthOrYear);
            if (newMonth !== this.activeDate.getMonth()) {
                this.loadDays();
            }
            const newDate = new Date(this.activeDate.getFullYear(), newMonth, 1);
            this.setActiveDateFocus(newDate, true);
        }
    }
    /**
     * This method sets focus for months/years depending on the loaded datepicker table
     */
    setFocusForCurrentMonthOryear() {
        const datePickerBody = $('.bs-datepicker-body');
        if (datePickerBody.find('table.months').length > 0) {
            this.loadMonths();
            this.setActiveMonthFocus(this.activeDate, true);
        }
        else if (datePickerBody.find('table.years').length > 0) {
            this.loadYears();
            this.setActiveYearFocus(this.activeDate, true);
        }
    }
    /**
     * This method sets focus for months/years/days depending on the loaded datepicker table
     */
    setFocusForDate(count) {
        const datePickerBody = $('.bs-datepicker-body');
        if (datePickerBody.find('table.months').length > 0) {
            this.loadMonths();
            const newDate = new Date(this.activeDate.getFullYear() + count, 0, this.activeDate.getDate());
            this.setActiveMonthFocus(newDate, true);
        }
        else if (datePickerBody.find('table.years').length > 0) {
            this.loadYears();
            const startYear = datePickerBody.find('table.years span').first().text();
            const newDate = new Date(_.parseInt(startYear), this.activeDate.getMonth(), this.activeDate.getDate());
            this.setActiveYearFocus(newDate, true);
        }
        else if (datePickerBody.find('table.days').length > 0) {
            this.loadDays();
            const newDate = new Date(this.activeDate.getFullYear(), this.activeDate.getMonth() + count, 1);
            this.setActiveDateFocus(newDate, true);
        }
    }
    /**
     * This method is used to add keyboard events while opening the date picker
     * @param scope - scope of the date/datetime widget
     * @param isDateTime - boolean value represents the loaded widget is date or datetime
     */
    addDatepickerKeyboardEvents(scope, isDateTime) {
        this.keyEventPluginInstance = scope.keyEventPlugin.constructor;
        this.elementScope = scope;
        const dateContainer = document.querySelector(`.${scope.dateContainerCls}`);
        setAttr(dateContainer, 'tabindex', '0');
        dateContainer.onkeydown = (event) => {
            const action = this.keyEventPluginInstance.getEventFullKey(event);
            // Check for Shift+Tab key or Tab key or escape
            if (action === 'shift.tab' || action === 'escape' || (action === 'tab' && !isDateTime)) {
                this.elementScope.hideDatepickerDropdown();
                const displayInputElem = this.elementScope.nativeElement.querySelector('.display-input');
                setTimeout(() => displayInputElem.focus());
            }
            if (action === 'tab' && isDateTime) {
                this.bsDatePickerDirective.hide();
                this.elementScope.toggleTimePicker(true);
            }
        };
        this.loadDays();
        this.setActiveDateFocus(this.activeDate);
    }
    /**
     * This method is used to add tabindex, keybord and mouse events for days
     */
    loadDays() {
        setTimeout(() => {
            $('.bs-datepicker-body').attr('tabindex', '0');
            $('[bsdatepickerdaydecorator]').not('.is-other-month').attr('tabindex', '0');
            this.addKeyBoardEventsForDays();
            this.addDatepickerMouseEvents();
        });
    }
    /**
     * This method sets keyboard events for days
     */
    addKeyBoardEventsForDays() {
        const datePickerBody = $('.bs-datepicker-body');
        datePickerBody.keydown((event) => {
            const action = this.keyEventPluginInstance.getEventFullKey(event);
            let newdate;
            if (action === 'arrowdown') {
                event.preventDefault();
                newdate = moment(this.activeDate).add(+7, 'days').toDate();
                this.setActiveDateFocus(newdate);
            }
            else if (action === 'arrowup') {
                event.preventDefault();
                newdate = moment(this.activeDate).add(-7, 'days').toDate();
                this.setActiveDateFocus(newdate);
            }
            else if (action === 'arrowleft') {
                newdate = moment(this.activeDate).add(-1, 'days').toDate();
                this.setActiveDateFocus(newdate);
            }
            else if (action === 'arrowright') {
                newdate = moment(this.activeDate).add(+1, 'days').toDate();
                this.setActiveDateFocus(newdate);
            }
            else if (action === 'pageup') {
                event.preventDefault();
                newdate = moment(this.activeDate).add(-1, 'month').toDate();
                this.setActiveDateFocus(newdate);
            }
            else if (action === 'pagedown') {
                event.preventDefault();
                newdate = moment(this.activeDate).add(+1, 'month').toDate();
                this.setActiveDateFocus(newdate);
            }
            else if (action === 'control.arrowup') {
                // clicking on table header month name to load list of months
                $('.bs-datepicker-head .current').first().click();
                this.loadMonths();
                this.setActiveMonthFocus(this.activeDate);
            }
            else if (action === 'enter') {
                if ($(document.activeElement).hasClass('disabled')) {
                    return;
                }
                $(document.activeElement).click();
                this.elementScope.hideDatepickerDropdown();
                const displayInputElem = this.elementScope.nativeElement.querySelector('.display-input');
                setTimeout(() => displayInputElem.focus());
            }
        });
    }
    /**
     * This method is used to add tabindex, keybord and mouse events for months
     */
    loadMonths() {
        setTimeout(() => {
            const datePickerBody = $('.bs-datepicker-body');
            datePickerBody.attr('tabindex', '0');
            datePickerBody.find('table.months span').attr('tabindex', '0');
            this.addKeyBoardEventsForMonths();
            this.addDatepickerMouseEvents();
        });
    }
    /**
     * This method sets keyboard events for months
     */
    addKeyBoardEventsForMonths() {
        $('.bs-datepicker-body').keydown((event) => {
            const action = this.keyEventPluginInstance.getEventFullKey(event);
            let newdate;
            if (action === 'arrowdown') {
                event.preventDefault();
                newdate = moment(this.activeDate).add(+3, 'month').toDate();
                this.setActiveMonthFocus(newdate);
            }
            else if (action === 'arrowup') {
                event.preventDefault();
                newdate = moment(this.activeDate).add(-3, 'month').toDate();
                this.setActiveMonthFocus(newdate);
            }
            else if (action === 'arrowleft') {
                newdate = moment(this.activeDate).add(-1, 'month').toDate();
                this.setActiveMonthFocus(newdate);
            }
            else if (action === 'arrowright') {
                newdate = moment(this.activeDate).add(+1, 'month').toDate();
                this.setActiveMonthFocus(newdate);
            }
            else if (action === 'control.arrowup') {
                // clicking on table header year to load list of years
                $('.bs-datepicker-head .current').first().click();
                this.loadYears();
                this.setActiveYearFocus(this.activeDate);
            }
            else if (action === 'control.arrowdown' || action === 'enter') {
                if ($(document.activeElement).parent().hasClass('disabled')) {
                    return;
                }
                $(document.activeElement).click();
                this.loadDays();
                const newDate = new Date(this.activeDate.getFullYear(), this.activeDate.getMonth(), 1);
                this.setActiveDateFocus(newDate);
            }
        });
    }
    /**
     * This method is used to add tabindex, keybord and mouse events for years
     */
    loadYears() {
        setTimeout(() => {
            const datePickerBody = $('.bs-datepicker-body');
            datePickerBody.attr('tabindex', '0');
            datePickerBody.find('table.years span').attr('tabindex', '0');
            this.addKeyBoardEventsForYears();
            this.addDatepickerMouseEvents();
        });
    }
    /**
     * This method is used to set focus for active month
     */
    setActiveMonthFocus(newDate, isMoouseEvent) {
        const newMonth = months[newDate.getMonth()];
        // check for keyboard event
        if (!isMoouseEvent) {
            if (newDate.getFullYear() < this.activeDate.getFullYear()) {
                this.goToOtherMonthOryear('previous', 'month');
            }
            else if (newDate.getFullYear() > this.activeDate.getFullYear()) {
                this.goToOtherMonthOryear('next', 'month');
            }
        }
        setTimeout(() => {
            $(`.bs-datepicker-body table.months span:contains(${newMonth})`).focus();
            this.activeDate = newDate;
        });
    }
    /**
     * This method sets keyboard events for years
     */
    addKeyBoardEventsForYears() {
        $('.bs-datepicker-body').keydown((event) => {
            const action = this.keyEventPluginInstance.getEventFullKey(event);
            let newdate;
            if (action === 'arrowdown') {
                event.preventDefault();
                newdate = moment(this.activeDate).add(+4, 'year').toDate();
                this.setActiveYearFocus(newdate);
            }
            else if (action === 'arrowup') {
                event.preventDefault();
                newdate = moment(this.activeDate).add(-4, 'year').toDate();
                this.setActiveYearFocus(newdate);
            }
            else if (action === 'arrowleft') {
                newdate = moment(this.activeDate).add(-1, 'year').toDate();
                this.setActiveYearFocus(newdate);
            }
            else if (action === 'arrowright') {
                newdate = moment(this.activeDate).add(+1, 'year').toDate();
                this.setActiveYearFocus(newdate);
            }
            else if (action === 'control.arrowdown' || action === 'enter') {
                if ($(document.activeElement).parent().hasClass('disabled')) {
                    return;
                }
                $(document.activeElement).click();
                this.loadMonths();
                this.setActiveMonthFocus(this.activeDate);
            }
        });
    }
    /**
     * This method is used to set focus for active year
     */
    setActiveYearFocus(newDate, isMouseEvent) {
        const newYear = newDate.getFullYear();
        const datePickerYears = $('.bs-datepicker-body table.years span');
        const startYear = datePickerYears.first().text();
        const endYear = datePickerYears.last().text();
        // check for keyboard event
        if (!isMouseEvent) {
            if (newDate.getFullYear() < _.parseInt(startYear)) {
                this.goToOtherMonthOryear('previous', 'year');
            }
            else if (newDate.getFullYear() > _.parseInt(endYear)) {
                this.goToOtherMonthOryear('next', 'year');
            }
        }
        setTimeout(() => {
            $(`.bs-datepicker-body table.years span:contains(${newYear})`).focus();
            this.activeDate = newDate;
        });
    }
    /**
     * This method sets focus for timepicker first input field(hours field) while opening time picker
     * @param scope - scope of the time picker widget
     */
    focusTimePickerPopover(scope) {
        this.keyEventPluginInstance = scope.keyEventPlugin.constructor;
        this.elementScope = scope;
        // setTimeout is used so that by then time input has the updated value. focus is setting back to the input field
        this.elementScope.ngZone.runOutsideAngular(() => {
            setTimeout(() => {
                $('timepicker .form-group').first().find('> input.form-control').focus();
            });
        });
    }
    /**
     * This function sets the keyboard events to Timepicker popup
     */
    bindTimePickerKeyboardEvents() {
        setTimeout(() => {
            const $timepickerPopup = $('body').find('> bs-dropdown-container timepicker');
            $timepickerPopup.attr('tabindex', 0);
            this.addEventsOnTimePicker($timepickerPopup);
        });
    }
    /**
     * This function checks whether the given date is valid or not
     * @returns boolean value, true if date is valid else returns false
     */
    isValidDate(date) {
        return date && date instanceof Date && !isNaN(date.getTime());
    }
    /**
     * This function checks whether the given time is valid or not
     */
    timeFormatValidation() {
        const enteredDate = $(this.nativeElement).find('input').val();
        const newVal = getNativeDateObject(enteredDate, { meridians: this.meridians, pattern: this.datepattern });
        if (!this.formatValidation(newVal, enteredDate)) {
            return;
        }
        this.invalidDateTimeFormat = false;
        this.invokeOnChange(this.datavalue, undefined, false);
    }
    updateTimeValue(value, fieldName) {
        if (value.length < 2) {
            setTimeout(() => {
                fieldName === 'minutes' ? this.bsTimePicker.updateMinutes('00') : this.bsTimePicker.updateSeconds('00');
            }, 500);
        }
        else {
            fieldName === 'minutes' ? this.bsTimePicker.updateMinutes('00') : this.bsTimePicker.updateSeconds('00');
        }
    }
    /**
     * This function sets the events to given element
     * @param $el - element on which the event is added
     */
    addEventsOnTimePicker($el) {
        const inputFields = $el.find('.bs-timepicker-field');
        // WMS-19382: update minutes and seconds to 0 when we enter hour value
        inputFields.first().on('keyup', evt => {
            const hourValue = evt.target.value;
            _.forEach(inputFields, (field, index) => {
                if (evt.target !== field && field.value === '' && hourValue.length) {
                    const fieldName = index === 1 ? 'minutes' : 'seconds';
                    this.updateTimeValue(hourValue, fieldName);
                }
            });
        });
        this.removeKeyupListener = () => {
            inputFields.first().off('keyup');
        };
        $el.on('keydown', evt => {
            const $target = $(evt.target);
            const $parent = $target.parent();
            const elementScope = this.elementScope;
            const action = this.keyEventPluginInstance.getEventFullKey(evt);
            let stopPropogation, preventDefault;
            if (action === 'escape') {
                elementScope.hideTimepickerDropdown();
            }
            if ($target.hasClass('bs-timepicker-field')) {
                if ($parent.is(':first-child')) {
                    if (action === 'shift.tab' || action === 'enter' || action === 'escape') {
                        elementScope.setIsTimeOpen(false);
                        this.focus();
                        stopPropogation = true;
                        preventDefault = true;
                    }
                }
                else if ($parent.is(':last-child') || ($parent.next().next().find('button.disabled').length)) {
                    if (action === 'tab' || action === 'escape' || action === 'enter') {
                        elementScope.setIsTimeOpen(false);
                        this.focus();
                        stopPropogation = true;
                        preventDefault = true;
                    }
                }
                else {
                    if (action === 'enter' || action === 'escape') {
                        elementScope.setIsTimeOpen(false);
                        this.focus();
                        stopPropogation = true;
                        preventDefault = true;
                    }
                }
                if (stopPropogation) {
                    evt.stopPropagation();
                }
                if (preventDefault) {
                    evt.preventDefault();
                }
                if (elementScope.mintime && elementScope.maxtime && !this.isValidDate(elementScope.bsTimeValue)) {
                    if (action === 'arrowdown') {
                        elementScope.bsTimeValue = elementScope.maxTime;
                    }
                    else if (action === 'arrowup') {
                        elementScope.bsTimeValue = elementScope.minTime;
                    }
                }
                if (action === 'tab') {
                    this.invalidDateTimeFormat = false;
                    this.invokeOnChange(this.datavalue, undefined, false);
                    const pattern = this.datepattern || this.timepattern;
                    if (getFormattedDate(elementScope.datePipe, elementScope.bsTimeValue, pattern) === elementScope.displayValue) {
                        $(this.nativeElement).find('.display-input').val(elementScope.displayValue);
                    }
                }
                if (action === 'arrowdown' || action === 'arrowup') {
                    this.timeFormatValidation();
                }
            }
            else if ($target.hasClass('btn-default')) {
                if (action === 'tab' || action === 'escape') {
                    elementScope.setIsTimeOpen(false);
                    this.focus();
                }
            }
        });
        $el.find('a').on('click', evt => {
            const elementScope = this.elementScope;
            const $target = $(evt.target);
            if (elementScope.mintime && elementScope.maxtime && !this.isValidDate(elementScope.bsTimeValue)) {
                if ($target.find('span').hasClass('bs-chevron-down')) {
                    elementScope.bsTimeValue = elementScope.maxTime;
                }
                else if ($target.find('span').hasClass('bs-chevron-up')) {
                    elementScope.bsTimeValue = elementScope.minTime;
                }
            }
            this.timeFormatValidation();
        });
    }
    updateFormat(pattern) {
        if (pattern === 'datepattern') {
            this._dateOptions.dateInputFormat = this.datepattern;
            this.showseconds = _.includes(this.datepattern, 's');
            this.ismeridian = _.includes(this.datepattern, 'h');
        }
        else if (pattern === 'timepattern') {
            this.showseconds = _.includes(this.timepattern, 's');
            this.ismeridian = _.includes(this.timepattern, 'h');
        }
    }
    getMobileInput() {
        return this.nativeElement.querySelector('.mobile-input');
    }
    onDateTimeInputBlur() {
        // removing the opacity on blur of the mobile widget
        let displayInputElem = this.getMobileInput();
        if (displayInputElem && !hasCordova()) {
            let children = this.nativeElement.children;
            for (let i = 0; i < children.length; i++) {
                children[i].classList.remove('add-opacity');
            }
            displayInputElem.classList.remove('remove-opacity');
        }
    }
    onDateTimeInputFocus(skipFocus = false) {
        let displayInputElem = this.getMobileInput();
        // toggling the classes to show and hide the native widget using opacity
        if (skipFocus && !hasCordova()) {
            let children = this.nativeElement.children;
            for (let i = 0; i < children.length; i++) {
                children[i].classList.add('add-opacity');
            }
            displayInputElem.classList.add('remove-opacity');
            return;
        }
        if (displayInputElem) {
            displayInputElem.focus();
            displayInputElem.click();
        }
    }
    onPropertyChange(key, nv, ov) {
        if (key === 'tabindex') {
            return;
        }
        if (key === 'required') {
            this._onChange(this.datavalue);
            return;
        }
        if (key === 'datepattern') {
            this.updateFormat(key);
        }
        else if (key === 'showweeks') {
            this._dateOptions.showWeekNumbers = nv;
        }
        else if (key === 'mindate') {
            this._dateOptions.minDate = (nv === CURRENT_DATE) ? this.mindate = new Date() : getDateObj(nv);
            this.minDateMaxDateValidationOnInput(this.datavalue);
        }
        else if (key === 'maxdate') {
            this._dateOptions.maxDate = (nv === CURRENT_DATE) ? this.maxdate = new Date() : getDateObj(nv);
            this.minDateMaxDateValidationOnInput(this.datavalue);
        }
        else if (key === 'excludedates' || key === 'excludedays') {
            if (this.excludedays) {
                this.excludedDaysToDisable = _.split(this.excludedays, ',').map((day) => {
                    return +day;
                });
            }
            if (this.excludedates) {
                this.excludedDatesToDisable = this.excludedates;
                if (isString(this.excludedates)) {
                    this.excludedDatesToDisable = _.split(this.excludedates, ',');
                }
                this.excludedDatesToDisable = this.excludedDatesToDisable.map(d => getDateObj(d));
            }
            this.minDateMaxDateValidationOnInput(this.datavalue);
        }
        else if (key === 'selectfromothermonth') {
            this._dateOptions.selectFromOtherMonth = nv;
        }
        else if (key === 'todaybutton') {
            this._dateOptions.showTodayButton = nv;
        }
        else if (key === 'clearbutton') {
            this._dateOptions.showClearButton = nv;
        }
        else if (key === 'todaybuttonlabel') {
            this._dateOptions.todayButtonLabel = this.i18nService.getLocalizedMessage(nv) || nv;
        }
        else if (key === 'clearbuttonlabel') {
            this._dateOptions.clearButtonLabel = this.i18nService.getLocalizedMessage(nv) || nv;
        }
        else {
            super.onPropertyChange(key, nv, ov);
        }
    }
    ngAfterViewInit() {
        super.ngAfterViewInit();
        this.containerTarget = getContainerTargetClass(this.nativeElement);
        this.isReadOnly = this.dataentrymode != 'undefined' && !this.isDataEntryModeEnabledOnInput(this.dataentrymode);
    }
    ngOnDestroy() {
        if (this.dateOnShowSubscription) {
            this.dateOnShowSubscription.unsubscribe();
        }
        super.ngOnDestroy();
    }
};
BaseDateTimeComponent.ctorParameters = () => [
    { type: Injector },
    null
];
__decorate([
    ViewChild(BsDropdownDirective),
    __metadata("design:type", Object)
], BaseDateTimeComponent.prototype, "bsDropdown", void 0);
__decorate([
    ViewChild(TimepickerComponent),
    __metadata("design:type", Object)
], BaseDateTimeComponent.prototype, "bsTimePicker", void 0);
BaseDateTimeComponent = __decorate([
    Directive(),
    __metadata("design:paramtypes", [Injector, Object])
], BaseDateTimeComponent);

const dateProps = new Map([
    ['autofocus', PROP_BOOLEAN],
    ['class', PROP_STRING],
    ['dataentrymode', Object.assign({ value: 'default' }, PROP_STRING)],
    ['datavaluesource', PROP_ANY],
    ['datavalue', PROP_STRING],
    ['datepattern', PROP_STRING],
    ['disabled', PROP_BOOLEAN],
    ['excludedays', PROP_STRING],
    ['excludedates', PROP_STRING],
    ['hint', PROP_STRING],
    ['maxdate', PROP_STRING],
    ['mindate', PROP_STRING],
    ['name', PROP_STRING],
    ['outputformat', Object.assign({ value: 'yyyy-MM-dd' }, PROP_STRING)],
    ['placeholder', Object.assign({ value: 'Select Date' }, PROP_STRING)],
    ['readonly', PROP_BOOLEAN],
    ['required', PROP_BOOLEAN],
    ['shortcutkey', PROP_STRING],
    ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
    ['showdropdownon', Object.assign({ value: 'default' }, PROP_STRING)],
    ['showbuttonbar', Object.assign({ value: true }, PROP_BOOLEAN)],
    ['showindevice', Object.assign({ displayType: 'inline-block', value: 'all' }, PROP_STRING)],
    ['showweeks', Object.assign({ value: false }, PROP_BOOLEAN)],
    ['tabindex', Object.assign({ value: 0 }, PROP_NUMBER)],
    ['selectfromothermonth', Object.assign({ value: true }, PROP_BOOLEAN)],
    ['todaybutton', Object.assign({ value: true }, PROP_BOOLEAN)],
    ['clearbutton', Object.assign({ value: true }, PROP_BOOLEAN)],
    ['todaybuttonlabel', Object.assign({ value: 'LABEL_TODAY_DATE' }, PROP_STRING)],
    ['clearbuttonlabel', Object.assign({ value: 'LABEL_CLEAR_DATE' }, PROP_STRING)]
]);
const registerProps = () => {
    register('wm-date', dateProps);
    registerFormWidget(FormWidgetType.DATE, new Map(dateProps));
};

var DateComponent_1;
const CURRENT_DATE$1 = 'CURRENT_DATE';
const DEFAULT_CLS = 'app-date input-group';
const WIDGET_CONFIG = {
    widgetType: 'wm-date',
    hostClass: DEFAULT_CLS
};
let DateComponent = DateComponent_1 = class DateComponent extends BaseDateTimeComponent {
    // TODO use BsLocaleService to set the current user's locale to see the localized labels
    constructor(inj, cdRef, appDefaults, evtMngrPlugins) {
        super(inj, WIDGET_CONFIG);
        this.cdRef = cdRef;
        this.appDefaults = appDefaults;
        this.isEnterPressedOnDateInput = false;
        styler(this.nativeElement, this);
        // KeyEventsPlugin
        this.keyEventPlugin = evtMngrPlugins[1];
        this.dateContainerCls = `app-date-${this.widgetId}`;
        this._dateOptions.containerClass = `app-date ${this.dateContainerCls}`;
        this._dateOptions.showWeekNumbers = false;
        this.datepattern = this.appDefaults.dateFormat || getDisplayDateTimeFormat(FormWidgetType.DATE);
        this.updateFormat('datepattern');
    }
    get timestamp() {
        return this.bsDataValue ? this.bsDataValue.valueOf() : undefined;
    }
    get dateInputFormat() {
        return this._dateOptions.dateInputFormat || 'yyyy-MM-dd';
    }
    get displayValue() {
        return getFormattedDate(this.datePipe, this.bsDataValue, this.dateInputFormat) || '';
    }
    get nativeDisplayValue() {
        return getFormattedDate(this.datePipe, this.bsDataValue, 'yyyy-MM-dd') || '';
    }
    get datavalue() {
        return getFormattedDate(this.datePipe, this.bsDataValue, this.outputformat) || '';
    }
    // Todo[Shubham]: needs to be redefined
    // sets the dataValue and computes the display model values
    set datavalue(newVal) {
        if (newVal === CURRENT_DATE$1) {
            this.bsDataValue = new Date();
        }
        else {
            this.bsDataValue = newVal ? getDateObj(newVal) : undefined;
        }
        // update the previous datavalue.
        this.invokeOnChange(this.datavalue, undefined, true);
        this.cdRef.detectChanges();
    }
    /**
     * This is an internal method triggered when the date input changes
     */
    onDisplayDateChange($event, isNativePicker) {
        if (this.isEnterPressedOnDateInput) {
            this.isEnterPressedOnDateInput = false;
            return;
        }
        const newVal = getDateObj($event.target.value, { pattern: this.datepattern });
        // date pattern validation
        // if invalid pattern is entered, device is showing an error.
        if (!this.formatValidation(newVal, $event.target.value, isNativePicker)) {
            return;
        }
        // min date and max date validation in mobile view.
        // if invalid dates are entered, device is showing an alert.
        if (isNativePicker && this.minDateMaxDateValidationOnInput(newVal, $event, this.displayValue, isNativePicker)) {
            return;
        }
        this.setDataValue(newVal);
    }
    // sets the dataValue and computes the display model values
    setDataValue(newVal) {
        this.invalidDateTimeFormat = false;
        // min date and max date validation in web.
        // if invalid dates are entered, device is showing validation message.
        this.minDateMaxDateValidationOnInput(newVal);
        if (getFormattedDate(this.datePipe, newVal, this.dateInputFormat) === this.displayValue) {
            $(this.nativeElement).find('.display-input').val(this.displayValue);
        }
        if (newVal) {
            this.bsDataValue = newVal;
        }
        else {
            this.bsDataValue = undefined;
        }
        this.invokeOnChange(this.datavalue, {}, true);
    }
    onDatePickerOpen() {
        this.isOpen = true;
        this.bsDataValue ? this.activeDate = this.bsDataValue : this.activeDate = new Date();
        if (!this.bsDataValue) {
            this.hightlightToday();
        }
        // We are using the two input tags(To maintain the modal and proxy modal) for the date control.
        // So actual bootstrap input target width we made it to 0, so bootstrap calculating the calendar container top position improperly.
        // To fix the container top position set the width 1px;
        this.$element.find('.model-holder').width('1px');
        this.addDatepickerKeyboardEvents(this, false);
        adjustContainerPosition($('bs-datepicker-container'), this.nativeElement, this.bsDatePickerDirective._datepicker);
        adjustContainerRightEdges($('bs-datepicker-container'), this.nativeElement, this.bsDatePickerDirective._datepicker);
    }
    onInputBlur($event) {
        if (!$($event.relatedTarget).hasClass('current-date')) {
            this.invokeOnTouched();
            this.invokeEventCallback('blur', { $event });
        }
    }
    hideDatepickerDropdown() {
        this.invokeOnTouched();
        this.isOpen = false;
        this.isEnterPressedOnDateInput = false;
        if (this.deregisterEventListener) {
            this.deregisterEventListener();
        }
    }
    // change and blur events are added from the template
    handleEvent(node, eventName, callback, locals) {
        if (!_.includes(['blur', 'focus', 'change', 'click'], eventName)) {
            super.handleEvent(node, eventName, callback, locals);
        }
    }
    /**
     * This is an internal method used to toggle the dropdown of the date widget
     */
    toggleDpDropdown($event) {
        if (isMobile()) {
            this.onDateTimeInputFocus();
            return;
        }
        if ($event.type === 'click') {
            this.invokeEventCallback('click', { $event: $event });
        }
        if ($event.target && $($event.target).is('input') && !(this.isDropDownDisplayEnabledOnInput(this.showdropdownon))) {
            $event.stopPropagation();
            return;
        }
        this.bsDatePickerDirective.toggle();
        this.addBodyClickListener(this.bsDatePickerDirective.isOpen);
    }
    addBodyClickListener(skipListener) {
        if (!skipListener) {
            return;
        }
        const bodyElement = document.querySelector('body');
        setTimeout(() => {
            const bsDateContainerElement = bodyElement.querySelector(`.${this.dateContainerCls}`);
            this.deregisterEventListener = addEventListenerOnElement(bodyElement, bsDateContainerElement, this.nativeElement, 'click', this.isDropDownDisplayEnabledOnInput(this.showdropdownon), () => {
                this.isOpen = false;
            }, 0 /* ONCE */, true);
        }, 350);
    }
    /**
     * This is an internal method triggered when pressing key on the date input
     */
    onDisplayKeydown(event) {
        if (this.isDropDownDisplayEnabledOnInput(this.showdropdownon)) {
            event.stopPropagation();
            const action = this.keyEventPlugin.constructor.getEventFullKey(event);
            if (action === 'enter' || action === 'arrowdown') {
                const newVal = getDateObj(event.target.value, { pattern: this.datepattern });
                event.preventDefault();
                const formattedDate = getFormattedDate(this.datePipe, newVal, this.dateInputFormat);
                const inputVal = event.target.value.trim();
                if (inputVal && this.datepattern === 'timestamp') {
                    if (!_.isNaN(inputVal) && _.parseInt(inputVal) !== formattedDate) {
                        this.invalidDateTimeFormat = true;
                        this.invokeOnChange(this.datavalue, event, false);
                    }
                }
                else if (inputVal && inputVal !== formattedDate) {
                    this.invalidDateTimeFormat = true;
                    this.invokeOnChange(this.datavalue, event, false);
                }
                else {
                    this.invalidDateTimeFormat = false;
                    this.isEnterPressedOnDateInput = true;
                    this.bsDatePickerDirective.bsValue = newVal;
                }
                this.toggleDpDropdown(event);
            }
            else {
                this.hideDatepickerDropdown();
            }
        }
        else {
            this.hideDatepickerDropdown();
        }
    }
    /**
     * This is an internal method triggered when the date selection changes
     */
    onDateChange(newVal) {
        const displayInputElem = this.nativeElement.querySelector('.display-input');
        if (this.isOpen) {
            setTimeout(() => displayInputElem.focus());
        }
        this.setDataValue(newVal);
    }
};
DateComponent.initializeProps = registerProps();
DateComponent.ctorParameters = () => [
    { type: Injector },
    { type: ChangeDetectorRef },
    { type: AppDefaults },
    { type: undefined, decorators: [{ type: Inject, args: [EVENT_MANAGER_PLUGINS,] }] }
];
__decorate([
    ViewChild(BsDatepickerDirective),
    __metadata("design:type", Object)
], DateComponent.prototype, "bsDatePickerDirective", void 0);
DateComponent = DateComponent_1 = __decorate([
    Component({
        selector: '[wmDate]',
        template: "    <input class=\"form-control app-textbox app-dateinput display-input\"\n        aria-label=\"Select date\"\n        focus-target\n        type=\"text\"\n        [name]=\"name\"\n        [tabindex]=\"tabindex\"\n        [value]=\"displayValue\"\n        (click)=\"toggleDpDropdown($event)\"\n        (focus)=\"invokeOnFocus($event)\"\n        (blur)=\"onInputBlur($event)\"\n        [disabled]=\"disabled || readonly\"\n        [autofocus]=\"autofocus\"\n        [required]=\"required\"\n        [readOnly]=\"isReadOnly\"\n        [attr.placeholder]=\"placeholder\"\n        [attr.accesskey]=\"shortcutkey\"\n        (change)=\"onDisplayDateChange($event)\"\n        (keydown)=\"onDisplayKeydown($event)\">\n    <div style=\"width: 0;display: inline-block;\"\n        aria-label=\"datepicker dropdownmenu\"\n        aria-controls=\"date\">\n        <input class=\"model-holder\"\n            [container]=\"containerTarget || 'body'\"\n            [bsConfig]=\"_dateOptions\"\n            [isOpen]=\"isOpen\"\n            #datepicker=\"bsDatepicker\"\n            bsDatepicker\n            [daysDisabled]=\"excludedDaysToDisable\"\n            [datesDisabled]=\"excludedDatesToDisable\"\n            placement=\"bottom right\"\n            [isDisabled]=\"disabled || readonly\"\n            [bsValue]=\"bsDataValue\"\n            (onShown)=\"onDatePickerOpen()\"\n            (onHidden)=\"hideDatepickerDropdown()\"\n            (bsValueChange)=\"onDateChange($event)\">\n    </div>\n    <span class=\"input-group-btn\">\n        <button type=\"button\"\n            class=\"btn btn-default btn-time\"\n            [tabindex]=\"tabindex\"\n            [disabled]=\"disabled || readonly\"\n            aria-label=\"Select date\"\n            aria-haspopup=\"true\"\n            aria-expanded=\"false\"\n            (click)=\"toggleDpDropdown($event)\">\n            <i aria-hidden=\"true\"\n                class=\"app-icon wi wi-calendar\"></i>\n        </button>\n    </span>\n\n<ng-template [ngIf]=\"loadNativeDateInput\">\n    <input type=\"date\"\n        class=\"form-control app-textbox app-dateinput mobile-input\"\n        role=\"input\"\n        step=\"any\"\n        placeholder=\"yyyy-mm-dd\" [min]=\"mindate | date:'yyyy-MM-dd'\" [max]=\"maxdate | date:'yyyy-MM-dd'\"\n        [value]=\"nativeDisplayValue\"\n        [tabindex]=\"tabindex\"\n        [required]=\"required\"\n        [disabled]=\"disabled || readonly\"\n        (change)=\"onDisplayDateChange($event, true)\"\n        (focus)=\"onDateTimeInputFocus(true); invokeOnFocus($event);\"\n        (blur)=\"onDateTimeInputBlur(); invokeOnTouched($event)\">\n</ng-template>\n",
        providers: [
            provideAs(DateComponent_1, NG_VALUE_ACCESSOR, true),
            provideAs(DateComponent_1, NG_VALIDATORS, true),
            provideAsWidgetRef(DateComponent_1)
        ]
    }),
    __param(3, Inject(EVENT_MANAGER_PLUGINS)),
    __metadata("design:paramtypes", [Injector,
        ChangeDetectorRef,
        AppDefaults, Object])
], DateComponent);

const dateTimeProps = new Map([
    ['autofocus', PROP_BOOLEAN],
    ['class', PROP_STRING],
    ['dataentrymode', Object.assign({ value: 'default' }, PROP_STRING)],
    ['datavaluesource', PROP_ANY],
    ['datavalue', PROP_STRING],
    ['datepattern', PROP_STRING],
    ['disabled', PROP_BOOLEAN],
    ['excludedays', PROP_STRING],
    ['excludedates', PROP_STRING],
    ['hint', PROP_STRING],
    ['hourstep', Object.assign({ value: 1 }, PROP_NUMBER)],
    ['maxdate', PROP_STRING],
    ['mindate', PROP_STRING],
    ['minutestep', Object.assign({ value: 15 }, PROP_NUMBER)],
    ['name', PROP_STRING],
    ['outputformat', Object.assign({ value: 'timestamp' }, PROP_STRING)],
    ['placeholder', Object.assign({ value: 'Select date time' }, PROP_STRING)],
    ['readonly', PROP_BOOLEAN],
    ['required', PROP_BOOLEAN],
    ['secondsstep', Object.assign({ value: 1 }, PROP_NUMBER)],
    ['shortcutkey', PROP_STRING],
    ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
    ['showdropdownon', Object.assign({ value: 'default' }, PROP_STRING)],
    ['showbuttonbar', PROP_BOOLEAN],
    ['showindevice', Object.assign({ displayType: 'inline-block', value: 'all' }, PROP_STRING)],
    ['showweeks', PROP_BOOLEAN],
    ['tabindex', Object.assign({ value: 0 }, PROP_NUMBER)],
    ['timestamp', PROP_STRING],
    ['selectfromothermonth', Object.assign({ value: true }, PROP_BOOLEAN)],
    ['todaybutton', Object.assign({ value: true }, PROP_BOOLEAN)],
    ['clearbutton', Object.assign({ value: true }, PROP_BOOLEAN)],
    ['todaybuttonlabel', Object.assign({ value: 'LABEL_TODAY_DATE' }, PROP_STRING)],
    ['clearbuttonlabel', Object.assign({ value: 'LABEL_CLEAR_DATE' }, PROP_STRING)]
]);
const registerProps$1 = () => {
    register('wm-datetime', dateTimeProps);
    registerFormWidget(FormWidgetType.DATETIME, new Map(dateTimeProps));
    registerFormWidget(FormWidgetType.TIMESTAMP, new Map(dateTimeProps));
};

var DatetimeComponent_1;
const DEFAULT_CLS$1 = 'app-datetime input-group';
const WIDGET_CONFIG$1 = { widgetType: 'wm-datetime', hostClass: DEFAULT_CLS$1 };
const CURRENT_DATE$2 = 'CURRENT_DATE';
const ɵ0 = getTimepickerConfig;
let DatetimeComponent = DatetimeComponent_1 = class DatetimeComponent extends BaseDateTimeComponent {
    constructor(inj, ngZone, cdRef, appDefaults, evtMngrPlugins) {
        super(inj, WIDGET_CONFIG$1);
        this.ngZone = ngZone;
        this.cdRef = cdRef;
        this.appDefaults = appDefaults;
        this.isEnterPressedOnDateInput = false;
        /**
         * This property checks if the timePicker is Open
         */
        this.isTimeOpen = false;
        /**
         * This property checks if the datePicker is Open
         */
        this.isDateOpen = false;
        /**
         * This property is set to TRUE if the time component value is set to CURRENT_TIME; In this case the timer keeps changing the time value until the widget is available.
         */
        this.isCurrentDate = false;
        this._debouncedOnChange = _.debounce(this.invokeOnChange, 10);
        this.registerDestroyListener(() => this.clearTimeInterval());
        styler(this.nativeElement, this);
        // KeyEventsPlugin
        this.keyEventPlugin = evtMngrPlugins[1];
        this.dateContainerCls = `app-date-${this.widgetId}`;
        this._dateOptions.containerClass = `app-date ${this.dateContainerCls}`;
        this._dateOptions.showWeekNumbers = false;
        this.datepattern = this.appDefaults.dateTimeFormat || getDisplayDateTimeFormat(FormWidgetType.DATETIME);
        this.updateFormat('datepattern');
    }
    get timestamp() {
        return this.proxyModel ? this.proxyModel.valueOf() : undefined;
    }
    get dateInputFormat() {
        return this._dateOptions.dateInputFormat || 'yyyy-MM-ddTHH:mm:ss';
    }
    /**
     * The displayValue is the display value of the bsDateTimeValue after applying the datePattern on it.
     * @returns {any|string}
     */
    get displayValue() {
        return getFormattedDate(this.datePipe, this.proxyModel, this.dateInputFormat) || '';
    }
    get nativeDisplayValue() {
        return getFormattedDate(this.datePipe, this.proxyModel, 'yyyy-MM-ddTHH:mm:ss') || '';
    }
    get datavalue() {
        if (this.isCurrentDate && !this.proxyModel) {
            return CURRENT_DATE$2;
        }
        return getFormattedDate(this.datePipe, this.proxyModel, this.outputformat);
    }
    /**Todo[Shubham]: needs to be redefined
     * This property sets the default value for the date selection
     */
    set datavalue(newVal) {
        if (newVal === CURRENT_DATE$2) {
            this.isCurrentDate = true;
            this.setTimeInterval();
        }
        else {
            this.proxyModel = newVal ? getDateObj(newVal) : undefined;
            this.clearTimeInterval();
            this.isCurrentDate = false;
        }
        this.bsTimeValue = this.bsDateValue = this.proxyModel;
        this.cdRef.detectChanges();
    }
    /**
     * This is an internal method used to maintain a time interval to update the time model when the data value is set to CURRENT_TIME
     */
    setTimeInterval() {
        if (this.timeinterval) {
            return;
        }
        this.timeinterval = setInterval(() => {
            const currentTime = new Date();
            this.onModelUpdate(currentTime);
        }, 1000);
    }
    /**
     * This is an internal method used to clear the time interval created
     */
    clearTimeInterval() {
        if (this.timeinterval) {
            clearInterval(this.timeinterval);
            this.timeinterval = null;
        }
    }
    /**
     * This is an internal method to toggle the time picker
     */
    toggleTimePicker(newVal, $event) {
        if (isMobile()) {
            this.onDateTimeInputFocus();
            return;
        }
        this.isTimeOpen = newVal;
        if ($event && $event.type === 'click') {
            this.invokeEventCallback('click', { $event: $event });
        }
        this.addTimepickerClickListener(this.isTimeOpen);
    }
    addTimepickerClickListener(skipListener) {
        if (!skipListener) {
            return;
        }
        const bodyElement = document.querySelector('body');
        setTimeout(() => {
            const dropdownElement = $(bodyElement).find('>bs-dropdown-container .dropdown-menu').get(0);
            this.deregisterTimepickeEventListener = addEventListenerOnElement(bodyElement, dropdownElement, this.nativeElement, 'click', this.isDropDownDisplayEnabledOnInput(this.showdropdownon), () => {
                this.toggleTimePicker(false);
            }, 0 /* ONCE */, true);
        }, 350);
    }
    /**
     * This function sets the value isOpen/isTimeOpen (i.e when datepicker popup is closed) based on widget type(i.e  DateTime, Time)
     * @param val - isOpen/isTimeOpen is set based on the timepicker popup is open/closed
     */
    setIsTimeOpen(val) {
        this.isTimeOpen = val;
    }
    hideTimepickerDropdown() {
        this.invokeOnTouched();
        this.toggleTimePicker(false);
        if (this.deregisterTimepickeEventListener) {
            this.deregisterTimepickeEventListener();
        }
    }
    /**
     * This is an internal method to add a click listener once the time dropdown is open
     */
    onTimepickerOpen() {
        // adding class for time widget dropdown menu
        const tpElements = document.querySelectorAll('timepicker');
        _.forEach(tpElements, (element) => {
            addClass(element.parentElement, 'app-datetime', true);
        });
        this.bsDatePickerDirective.hide();
        this.focusTimePickerPopover(this);
        this.bindTimePickerKeyboardEvents();
        adjustContainerPosition($('bs-dropdown-container'), this.nativeElement, this.bsDropdown._dropdown, $('bs-dropdown-container .dropdown-menu'));
        adjustContainerRightEdges($('bs-dropdown-container'), this.nativeElement, this.bsDropdown._dropdown, $('bs-dropdown-container .dropdown-menu'));
    }
    onDatePickerOpen() {
        this.isDateOpen = !this.isDateOpen;
        this.toggleTimePicker(false);
        // We are using the two input tags(To maintain the modal and proxy modal) for the date control.
        // So actual bootstrap input target width we made it to 0 so bootstrap calculating the calendar container top position impropery.
        // To fix the container top position set the width 1px;
        this.$element.find('.model-holder').width('1px');
        this.bsDateValue ? this.activeDate = this.bsDateValue : this.activeDate = new Date();
        if (!this.bsDateValue) {
            this.hightlightToday();
        }
        this.addDatepickerKeyboardEvents(this, true);
        adjustContainerPosition($('bs-datepicker-container'), this.nativeElement, this.bsDatePickerDirective._datepicker);
    }
    /**
     * This is an internal method to update the model
     */
    onModelUpdate(newVal, type) {
        if (type === 'date') {
            this.invalidDateTimeFormat = false;
            if (getFormattedDate(this.datePipe, newVal, this.dateInputFormat) === this.displayValue) {
                $(this.nativeElement).find('.display-input').val(this.displayValue);
            }
        }
        // min date and max date validation in web.
        // if invalid dates are entered, device is showing validation message.
        this.minDateMaxDateValidationOnInput(newVal);
        if (!newVal) {
            // Set timevalue as 00:00:00 if we remove any one from hours/minutes/seconds field in timepicker after selecting date
            if (this.bsDateValue && this.bsTimePicker && (this.bsTimePicker.hours === '' || this.bsTimePicker.minutes === '' || this.bsTimePicker.seconds === '')) {
                this.bsDateValue = this.bsTimeValue = this.proxyModel = moment(this.bsDateValue).startOf('day').toDate();
            }
            else {
                this.bsDateValue = this.bsTimeValue = this.proxyModel = undefined;
            }
            this._debouncedOnChange(this.datavalue, {}, true);
            return;
        }
        if (type === 'date') {
            if (this.isDateOpen) {
                this.toggleTimePicker(true);
            }
        }
        this.proxyModel = newVal;
        if (this.proxyModel) {
            this.bsDateValue = this.bsTimeValue = this.proxyModel;
        }
        this._debouncedOnChange(this.datavalue, {}, true);
        this.cdRef.detectChanges();
    }
    /**
     * This is an internal method used to Prevent time picker close while changing time value
     */
    preventTpClose($event) {
        $event.stopImmediatePropagation();
    }
    /**
     * This is an internal method used to toggle the dropdown of the date widget
     */
    toggleDpDropdown($event) {
        if (isMobile()) {
            this.onDateTimeInputFocus();
            return;
        }
        if ($event.type === 'click') {
            this.invokeEventCallback('click', { $event: $event });
        }
        if ($event.target && $($event.target).is('input') && !(this.isDropDownDisplayEnabledOnInput(this.showdropdownon))) {
            $event.stopPropagation();
            return;
        }
        this.bsDatePickerDirective.toggle();
        this.addBodyClickListener(this.bsDatePickerDirective.isOpen);
    }
    addBodyClickListener(skipListener) {
        if (!skipListener) {
            return;
        }
        const bodyElement = document.querySelector('body');
        setTimeout(() => {
            const bsDateContainerElement = bodyElement.querySelector(`.${this.dateContainerCls}`);
            this.deregisterDatepickerEventListener = addEventListenerOnElement(bodyElement, bsDateContainerElement, this.nativeElement, 'click', this.isDropDownDisplayEnabledOnInput(this.showdropdownon), () => {
                this.bsDatePickerDirective.hide();
            }, 0 /* ONCE */, true);
        }, 350);
    }
    hideDatepickerDropdown() {
        this.isDateOpen = false;
        this.invokeOnTouched();
        this.bsDatePickerDirective.hide();
        this.isEnterPressedOnDateInput = false;
        if (this.deregisterDatepickerEventListener) {
            this.deregisterDatepickerEventListener();
        }
    }
    onDateChange($event, isNativePicker) {
        if (this.isEnterPressedOnDateInput) {
            this.isEnterPressedOnDateInput = false;
            return;
        }
        let newVal = $event.target.value.trim();
        newVal = newVal ? getNativeDateObject(newVal, { pattern: this.datepattern, meridians: this.meridians }) : undefined;
        // datetime pattern validation
        // if invalid pattern is entered, device is showing an error.
        if (!this.formatValidation(newVal, $event.target.value, isNativePicker)) {
            return;
        }
        // min date and max date validation in mobile view.
        // if invalid dates are entered, device is showing an alert.
        if (isNativePicker && this.minDateMaxDateValidationOnInput(newVal, $event, this.displayValue, isNativePicker)) {
            return;
        }
        this.invalidDateTimeFormat = false;
        this.onModelUpdate(newVal);
    }
    /**
     * This is an internal method triggered when pressing key on the datetime input
     */
    onDisplayKeydown(event) {
        if (this.isDropDownDisplayEnabledOnInput(this.showdropdownon)) {
            event.stopPropagation();
            let newVal = event.target.value.trim();
            const action = this.keyEventPlugin.constructor.getEventFullKey(event);
            if (action === 'enter' || action === 'arrowdown') {
                newVal = newVal ? getNativeDateObject(newVal, { pattern: this.datepattern, meridians: this.meridians }) : undefined;
                event.preventDefault();
                const formattedDate = getFormattedDate(this.datePipe, newVal, this.dateInputFormat);
                const inputVal = event.target.value.trim();
                if (inputVal && this.datepattern === 'timestamp') {
                    if (!_.isNaN(inputVal) && _.parseInt(inputVal) !== formattedDate) {
                        this.invalidDateTimeFormat = true;
                        this.invokeOnChange(this.datavalue, event, false);
                    }
                }
                else if (inputVal && inputVal !== formattedDate) {
                    this.invalidDateTimeFormat = true;
                    this.invokeOnChange(this.datavalue, event, false);
                }
                else {
                    this.invalidDateTimeFormat = false;
                    this.isEnterPressedOnDateInput = true;
                    this.bsDatePickerDirective.bsValue = newVal;
                }
                this.toggleDpDropdown(event);
            }
            else {
                this.hideDatepickerDropdown();
                this.hideTimepickerDropdown();
            }
        }
        else {
            this.hideDatepickerDropdown();
            this.hideTimepickerDropdown();
        }
    }
    isValid(event) {
        if (!event) {
            const enteredDate = $(this.nativeElement).find('input').val();
            const newVal = getNativeDateObject(enteredDate, { pattern: this.datepattern, meridians: this.meridians });
            if (!this.formatValidation(newVal, enteredDate)) {
                return;
            }
        }
    }
    // change and blur events are added from the template
    handleEvent(node, eventName, callback, locals) {
        if (!_.includes(['blur', 'focus', 'change', 'click'], eventName)) {
            super.handleEvent(node, eventName, callback, locals);
        }
    }
    onInputBlur($event) {
        if (!$($event.relatedTarget).hasClass('current-date')) {
            this.invokeOnTouched();
            this.invokeEventCallback('blur', { $event });
        }
    }
};
DatetimeComponent.initializeProps = registerProps$1();
DatetimeComponent.ctorParameters = () => [
    { type: Injector },
    { type: NgZone },
    { type: ChangeDetectorRef },
    { type: AppDefaults },
    { type: undefined, decorators: [{ type: Inject, args: [EVENT_MANAGER_PLUGINS,] }] }
];
__decorate([
    ViewChild(BsDatepickerDirective),
    __metadata("design:type", Object)
], DatetimeComponent.prototype, "bsDatePickerDirective", void 0);
DatetimeComponent = DatetimeComponent_1 = __decorate([
    Component({
        selector: '[wmDateTime]',
        template: "    <div dropdown\n        [isOpen]=\"isTimeOpen\"\n        autoClose=\"false\"\n        [container]=\"'body'\"\n        (onShown)=\"onTimepickerOpen()\"\n        style=\"display: inherit;\">\n        <input class=\"form-control app-textbox display-input\"\n            aria-label=\"Set the date and time\"\n            focus-target\n            [name]=\"name\"\n            [tabindex]=\"tabindex\"\n            type=\"text\"\n            [value]=\"displayValue\"\n            (click)=\"toggleDpDropdown($event)\"\n            (focus)=\"invokeOnFocus($event)\"\n            (blur)=\"onInputBlur($event)\"\n            (change)=\"onDateChange($event)\"\n            [autofocus]=\"autofocus\"\n            [disabled]=\"disabled || readonly || isCurrentDate\"\n            [required]=\"required\"\n            [readOnly]=\"isReadOnly\"\n            [attr.placeholder]=\"placeholder\"\n            [attr.accesskey]=\"shortcutkey\"\n            (keydown)=\"onDisplayKeydown($event)\">\n        <div style=\"width: 0;display: inline-block;\"\n            aria-label=\"datepicker dropdownmenu\"\n            aria-controls=\"date\">\n            <input class=\"model-holder\"\n                focus-target\n                [container]=\"containerTarget || 'body'\"\n                [bsConfig]=\"_dateOptions\"\n                [isOpen]=\"isDateOpen\"\n                 placement=\"bottom right\"\n                (onShown)=\"onDatePickerOpen()\"\n                (onHidden)=\"hideDatepickerDropdown()\"\n                [daysDisabled]=\"excludedDaysToDisable\"\n                [datesDisabled]=\"excludedDatesToDisable\"\n                #datepicker=bsDatepicker\n                bsDatepicker\n                [isDisabled]=\"disabled || readonly || isCurrentDate\"\n                [bsValue]=\"bsDateValue\"\n                (bsValueChange)=\"onModelUpdate($event, 'date')\">\n        </div>\n        <span class=\"input-group-btn\">\n            <button type=\"button\"\n                class=\"btn btn-default btn-date\"\n                [tabindex]=\"tabindex\"\n                [disabled]=\"disabled || readonly || isCurrentDate\"\n                aria-label=\"Select date\"\n                aria-haspopup=\"true\"\n                aria-expanded=\"false\"\n                (click)=\"toggleDpDropdown($event)\">\n                <i aria-hidden=\"true\"\n                    class=\"app-icon wi wi-calendar\"></i>\n            </button>\n            <button type=\"button\"\n                class=\"btn btn-default btn-time\"\n                [tabindex]=\"tabindex\"\n                [disabled]=\"disabled || readonly || isCurrentDate\"\n                aria-label=\"Select time\"\n                aria-haspopup=\"true\"\n                aria-expanded=\"false\"\n                (click)=\"toggleTimePicker(!isTimeOpen, $event)\">\n                <i aria-hidden=\"true\"\n                    class=\"app-icon wi wi-access-time\"></i>\n            </button>\n        </span>\n        <div *dropdownMenu\n            class=\"dropdown-menu\"\n            aria-label=\"timepicker dropdown\"\n            aria-controls=\"time\"\n            (click)=\"preventTpClose($event)\">\n            <timepicker class=\"model-holder\"\n                [showMeridian]=\"ismeridian\"\n                [readonlyInput]=\"disabled || readonly || isCurrentDate\"\n                [(ngModel)]=\"bsTimeValue\"\n                [min]=\"minTime\"\n                [max]=\"maxTime\"\n                [hourStep]=\"hourstep\"\n                [minuteStep]=\"minutestep\"\n                [secondsStep]=\"secondsstep\"\n                [mousewheel]=\"true\"\n                [arrowkeys]=\"true\"\n                (isValid)=\"isValid($event)\"\n                [showSeconds]=\"showseconds\"\n                (ngModelChange)=\"onModelUpdate($event, 'time')\"></timepicker>\n        </div>\n    </div>\n<ng-template [ngIf]=\"loadNativeDateInput\">\n    <input type=\"datetime-local\"\n        class=\"form-control app-textbox mobile-input\"\n        role=\"input\"\n        step=\"any\"\n        [value]=\"nativeDisplayValue\" [min]=\"mindate | date:'yyyy-MM-ddTHH:mm:ss'\" [max]=\"maxdate  | date:'yyyy-MM-ddTHH:mm:ss'\"\n        [tabindex]=\"tabindex\"\n        [required]=\"required\"\n        [disabled]=\"disabled || readonly || isCurrentDate\"\n        (change)=\"onDateChange($event, true)\"\n        (focus)=\"onDateTimeInputFocus(true); invokeOnFocus($event)\"\n        (blur)=\"onDateTimeInputBlur(); invokeOnTouched($event)\">\n</ng-template>\n",
        providers: [
            provideAs(DatetimeComponent_1, NG_VALUE_ACCESSOR, true),
            provideAs(DatetimeComponent_1, NG_VALIDATORS, true),
            provideAsWidgetRef(DatetimeComponent_1),
            { provide: TimepickerConfig, deps: [AbstractI18nService], useFactory: ɵ0 }
        ]
    }),
    __param(4, Inject(EVENT_MANAGER_PLUGINS)),
    __metadata("design:paramtypes", [Injector,
        NgZone,
        ChangeDetectorRef,
        AppDefaults, Object])
], DatetimeComponent);

const timeProps = new Map([
    ['autofocus', PROP_BOOLEAN],
    ['class', PROP_STRING],
    ['dataentrymode', Object.assign({ value: 'default' }, PROP_STRING)],
    ['datavaluesource', PROP_ANY],
    ['datavalue', PROP_STRING],
    ['disabled', PROP_BOOLEAN],
    ['hint', PROP_STRING],
    ['hourstep', Object.assign({ value: 1 }, PROP_NUMBER)],
    ['maxtime', PROP_STRING],
    ['mintime', PROP_STRING],
    ['minutestep', Object.assign({ value: 15 }, PROP_NUMBER)],
    ['name', PROP_STRING],
    ['outputformat', Object.assign({ value: 'HH:mm:ss' }, PROP_STRING)],
    ['placeholder', Object.assign({ value: 'Select time' }, PROP_STRING)],
    ['readonly', PROP_BOOLEAN],
    ['required', PROP_BOOLEAN],
    ['shortcutkey', PROP_STRING],
    ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
    ['showdropdownon', Object.assign({ value: 'default' }, PROP_STRING)],
    ['secondsstep', Object.assign({ value: 1 }, PROP_NUMBER)],
    ['tabindex', Object.assign({ value: 0 }, PROP_NUMBER)],
    ['timepattern', PROP_STRING],
    ['timestamp', PROP_STRING]
]);
const registerProps$2 = () => {
    register('wm-time', timeProps);
    registerFormWidget(FormWidgetType.TIME, new Map(timeProps));
};

var TimeComponent_1;
const CURRENT_TIME = 'CURRENT_TIME';
const DEFAULT_CLS$2 = 'input-group app-timeinput';
const WIDGET_CONFIG$2 = { widgetType: 'wm-time', hostClass: DEFAULT_CLS$2 };
const ɵ0$1 = getTimepickerConfig;
let TimeComponent = TimeComponent_1 = class TimeComponent extends BaseDateTimeComponent {
    constructor(inj, ngZone, appDefaults, evtMngrPlugins) {
        super(inj, WIDGET_CONFIG$2);
        this.ngZone = ngZone;
        this.appDefaults = appDefaults;
        /**
         * This is an internal property used to toggle the timepicker dropdown
         */
        this.status = { isopen: false };
        // KeyEventsPlugin
        this.keyEventPlugin = evtMngrPlugins[1];
        styler(this.nativeElement, this);
        /**
         * Destroy the timer once the date widget is gone
         */
        this.registerDestroyListener(() => this.clearTimeInterval());
        this.timepattern = this.appDefaults.timeFormat || getDisplayDateTimeFormat(FormWidgetType.TIME);
        this.updateFormat('timepattern');
    }
    get timestamp() {
        return this.bsTimeValue ? this.bsTimeValue.valueOf() : undefined;
    }
    get datavalue() {
        if (this.isCurrentTime && !this.bsTimeValue) {
            return CURRENT_TIME;
        }
        return getFormattedDate(this.datePipe, this.bsTimeValue, this.outputformat) || '';
    }
    /**Todo[Shubham]: needs to be redefined
     * This property sets the default value for the time selection
     */
    set datavalue(newVal) {
        if (newVal) {
            if (newVal === CURRENT_TIME) {
                this.isCurrentTime = true;
                this.setTimeInterval();
            }
            else {
                this.clearTimeInterval();
                this.bsTimeValue = getNativeDateObject(newVal);
                this.isCurrentTime = false;
                this.mintimeMaxtimeValidation();
            }
        }
        else {
            this.bsTimeValue = undefined;
            this.clearTimeInterval();
            this.isCurrentTime = false;
        }
        this.invokeOnChange(this.datavalue);
        $appDigest();
    }
    get displayValue() {
        return getFormattedDate(this.datePipe, this.bsTimeValue, this.timepattern) || '';
    }
    get nativeDisplayValue() {
        return getFormattedDate(this.datePipe, this.bsTimeValue, 'hh:mm:ss') || '';
    }
    onPropertyChange(key, nv, ov) {
        if (key === 'tabindex') {
            return;
        }
        if (key === 'timepattern') {
            this.updateFormat('timepattern');
        }
        if (key === 'mintime') {
            this.minTime = getNativeDateObject(nv); // TODO it is supposed to be time conversion, not to the day
            this.mintimeMaxtimeValidation();
        }
        else if (key === 'maxtime') {
            this.maxTime = getNativeDateObject(nv);
            this.mintimeMaxtimeValidation();
        }
        else {
            super.onPropertyChange(key, nv, ov);
        }
    }
    setValidateType(min, max, val) {
        if (this.timeNotInRange) {
            if (val < min) {
                this.validateType = 'mintime';
            }
            else if (val > max) {
                this.validateType = 'maxtime';
            }
        }
    }
    /**
     * This is an internal method used to validate mintime and maxtime
     */
    mintimeMaxtimeValidation() {
        this.timeNotInRange = this.minTime && this.maxTime && (this.bsTimeValue < this.minTime || this.bsTimeValue > this.maxTime);
        this.setValidateType(this.minTime, this.maxTime, this.bsTimeValue);
        this.invokeOnChange(this.datavalue, undefined, false);
    }
    /**
     * This is an internal method used to toggle the dropdown of the time widget
     */
    toggleDropdown($event) {
        if (isMobile()) {
            this.onDateTimeInputFocus();
            return;
        }
        if ($event.type === 'click') {
            this.invokeEventCallback('click', { $event: $event });
        }
        if ($event.target && $($event.target).is('input') && !(this.isDropDownDisplayEnabledOnInput(this.showdropdownon))) {
            return;
        }
        this.ngZone.run(() => {
            this.status.isopen = !this.status.isopen;
        });
        this.addBodyClickListener(this.status.isopen);
    }
    /**
     * This is an internal method used to Prevent time picker close while changing time value
     */
    preventTpClose($event) {
        $event.stopImmediatePropagation();
    }
    addBodyClickListener(skipListener) {
        if (!skipListener) {
            return;
        }
        const bodyElement = document.querySelector('body');
        setTimeout(() => {
            const dropdownElement = $(bodyElement).find('>bs-dropdown-container .dropdown-menu').get(0);
            this.deregisterEventListener = addEventListenerOnElement(bodyElement, dropdownElement, this.nativeElement, 'click', this.isDropDownDisplayEnabledOnInput(this.showdropdownon), () => {
                this.status.isopen = false;
            }, 0 /* ONCE */, true);
        }, 350);
    }
    /**
     * This is an internal method triggered when pressing key on the time input
     */
    onDisplayKeydown(event) {
        if (this.isDropDownDisplayEnabledOnInput(this.showdropdownon)) {
            event.stopPropagation();
            const action = this.keyEventPlugin.constructor.getEventFullKey(event);
            if (action === 'enter' || action === 'arrowdown') {
                event.preventDefault();
                this.toggleDropdown(event);
            }
            else {
                this.hideTimepickerDropdown();
            }
        }
        else {
            this.hideTimepickerDropdown();
        }
    }
    /**
     * This is an internal method triggered when the time input changes
     */
    onDisplayTimeChange($event) {
        const newVal = getNativeDateObject($event.target.value, { meridians: this.meridians });
        // time pattern validation
        // if invalid pattern is entered, device is showing an error.
        if (!this.formatValidation(newVal, $event.target.value)) {
            return;
        }
        this.invalidDateTimeFormat = false;
        this.onTimeChange(newVal);
    }
    onInputBlur($event) {
        if (!$($event.relatedTarget).hasClass('bs-timepicker-field')) {
            this.invokeOnTouched();
            this.invokeEventCallback('blur', { $event });
        }
    }
    /**
     * This is an internal method used to execute the on time change functionality
     */
    onTimeChange(newVal, isNativePicker) {
        let timeValue, timeInputValue, minTimeMeridian, maxTimeMeridian;
        // For nativePicker, newVal is event, get the dateobject from the event.
        if (isNativePicker) {
            newVal = getNativeDateObject(newVal.target.value);
        }
        if (newVal) {
            this.bsTimeValue = newVal;
            // if the newVal is valid but not in the given range then highlight the input field
            this.timeNotInRange = this.minTime && this.maxTime && (newVal < this.minTime || newVal > this.maxTime);
            this.setValidateType(this.minTime, this.maxTime, newVal);
        }
        else {
            // sometimes library is not returning the correct value when the min and max time are given, displaying the datavalue based on the value given by the user
            if (this.bsTimePicker && this.bsTimePicker.min && this.bsTimePicker.max) {
                minTimeMeridian = moment(new Date(this.bsTimePicker.min)).format('A');
                maxTimeMeridian = moment(new Date(this.bsTimePicker.max)).format('A');
                timeValue = this.bsTimePicker.hours + ':' + (this.bsTimePicker.minutes || 0) + ':' + (this.bsTimePicker.seconds || 0) + (this.bsTimePicker.showMeridian ? (' ' + minTimeMeridian) : '');
                timeInputValue = getNativeDateObject(timeValue);
                this.bsTimePicker.meridian = minTimeMeridian;
                this.timeNotInRange = (this.bsTimePicker.min > timeInputValue || this.bsTimePicker.max < timeInputValue);
                this.setValidateType(this.bsTimePicker.min, this.bsTimePicker.max, timeInputValue);
            }
            this.bsTimeValue = timeInputValue;
        }
        this.invokeOnTouched();
        this.invokeOnChange(this.datavalue, {}, true);
    }
    /**
     * This is an internal method used to maintain a time interval to update the time model when the data value is set to CURRENT_TIME
     */
    setTimeInterval() {
        if (this.timeinterval) {
            return;
        }
        this.timeinterval = setInterval(() => {
            const now = new Date();
            now.setSeconds(now.getSeconds() + 1);
            this.datavalue = CURRENT_TIME;
            this.onTimeChange(now);
        }, 1000);
    }
    /**
     * This is an internal method used to clear the time interval created
     */
    clearTimeInterval() {
        if (this.timeinterval) {
            clearInterval(this.timeinterval);
            this.timeinterval = null;
        }
    }
    /**
     * This function sets the value isOpen/isTimeOpen (i.e when datepicker popup is closed) based on widget type(i.e  DateTime, Time)
     * @param val - isOpen/isTimeOpen is set based on the timepicker popup is open/closed
     */
    setIsTimeOpen(val) {
        this.status.isopen = val;
    }
    // Change event is registered from the template, Prevent the framework from registering one more event
    handleEvent(node, eventName, eventCallback, locals) {
        if (!_.includes(['blur', 'focus', 'change', 'click'], eventName)) {
            super.handleEvent(node, eventName, eventCallback, locals);
        }
    }
    hideTimepickerDropdown() {
        this.invokeOnTouched();
        this.status.isopen = false;
        if (this.deregisterEventListener) {
            this.deregisterEventListener();
        }
        this.removeKeyupListener();
    }
    isValid(event) {
        if (!event) {
            const enteredDate = $(this.nativeElement).find('input').val();
            const newVal = getNativeDateObject(enteredDate, { meridians: this.meridians });
            if (!this.formatValidation(newVal, enteredDate)) {
                return;
            }
        }
    }
    /**
     * This is an internal method to add css class for dropdown while opening the time dropdown
     */
    onShown() {
        const tpElements = document.querySelectorAll('timepicker');
        _.forEach(tpElements, element => {
            addClass(element.parentElement, 'app-datetime', true);
        });
        this.focusTimePickerPopover(this);
        this.bindTimePickerKeyboardEvents();
        adjustContainerPosition($('bs-dropdown-container'), this.nativeElement, this.bsDropdown._dropdown, $('bs-dropdown-container .dropdown-menu'));
    }
};
TimeComponent.initializeProps = registerProps$2();
TimeComponent.ctorParameters = () => [
    { type: Injector },
    { type: NgZone },
    { type: AppDefaults },
    { type: undefined, decorators: [{ type: Inject, args: [EVENT_MANAGER_PLUGINS,] }] }
];
TimeComponent = TimeComponent_1 = __decorate([
    Component({
        selector: '[wmTime]',
        template: "    <div dropdown [isOpen]=\"status.isopen\" (onShown)=\"onShown()\" (onHidden)=\"hideTimepickerDropdown()\" [container]=\"'body'\" style=\"display: inherit\">\n        <input class=\"form-control app-textbox display-input\" aria-label=\"Set the time\"\n               focus-target\n               [tabindex]=\"tabindex\"\n               [name]=\"name\"\n               type=\"text\"\n               [value]=\"displayValue\"\n               [disabled]=\"disabled || readonly || isCurrentTime\"\n               [autofocus]=\"autofocus\"\n               [required]=\"required\"\n               [attr.placeholder]=\"placeholder\"\n               [attr.accesskey]=\"shortcutkey\"\n               [required]=\"required\"\n               [readOnly]=\"isReadOnly\"\n               (click)=\"toggleDropdown($event)\"\n               (focus)=\"invokeOnFocus($event)\"\n               (blur)=\"onInputBlur($event)\"\n               (change)=\"onDisplayTimeChange($event)\"\n               (keydown)=\"onDisplayKeydown($event)\">\n        <span class=\"input-group-btn dropdown-toggle\">\n          <button type=\"button\" class=\"btn btn-default btn-date\" [tabindex]=\"tabindex\" [disabled]=\"disabled || readonly || isCurrentTime\" aria-label=\"Select time\" aria-haspopup=\"true\" aria-expanded=\"false\" (click)=\"toggleDropdown($event)\"><i aria-hidden=\"true\" class=\"app-icon wi wi-access-time\"></i></button>\n        </span>\n        <div *dropdownMenu class=\"dropdown-menu\" (click)=\"preventTpClose($event)\">\n            <timepicker class=\"model-holder\"\n                        [showMeridian]=\"ismeridian\"\n                        [readonlyInput]=\"isDisabled\"\n                        [(ngModel)]=\"bsTimeValue\"\n                        [disabled]=\"disabled || readonly || isCurrentTime\"\n                        [min]=\"minTime\"\n                        [max]=\"maxTime\"\n                        [hourStep]=\"hourstep\"\n                        [minuteStep]=\"minutestep\"\n                        [secondsStep]=\"secondsstep\"\n                        [mousewheel]=\"true\"\n                        [arrowkeys]=\"true\"\n                        (isValid)=\"isValid($event)\"\n                        [showSeconds]=\"showseconds\" (ngModelChange)=\"onTimeChange($event)\"></timepicker>\n        </div>\n    </div>\n<ng-template [ngIf]=\"loadNativeDateInput\">\n    <input type=\"time\" class=\"form-control app-textbox display-input mobile-input\"\n           role=\"input\"\n           [value]=\"nativeDisplayValue\" [min]=\"mintime\" [max]=\"maxtime\"\n           [tabindex]=\"tabindex\"\n           [required]=\"required\"\n           [disabled]=\"disabled || readonly || isCurrentTime\"\n           (change)=\"onTimeChange($event, true)\"\n           (focus)=\"onDateTimeInputFocus(true); invokeOnFocus($event)\"\n           (blur)=\"onDateTimeInputBlur(); invokeOnTouched($event)\">\n</ng-template>\n",
        providers: [
            provideAs(TimeComponent_1, NG_VALUE_ACCESSOR, true),
            provideAs(TimeComponent_1, NG_VALIDATORS, true),
            provideAsWidgetRef(TimeComponent_1),
            { provide: TimepickerConfig, deps: [AbstractI18nService], useFactory: ɵ0$1 }
        ]
    }),
    __param(3, Inject(EVENT_MANAGER_PLUGINS)),
    __metadata("design:paramtypes", [Injector,
        NgZone,
        AppDefaults, Object])
], TimeComponent);

const components = [
    DateComponent,
    DatetimeComponent,
    TimeComponent
];
let EpochModule = class EpochModule {
};
EpochModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            BsDatepickerModule,
            BsDropdownModule,
            FormsModule,
            TimepickerModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], EpochModule);

/**
 * Generated bundle index. Do not edit.
 */

export { DateComponent, DatetimeComponent, EpochModule, TimeComponent, ɵ0, getTimepickerConfig as ɵa, BaseDateTimeComponent as ɵb, registerProps as ɵc, registerProps$1 as ɵd, registerProps$2 as ɵe };
//# sourceMappingURL=index.js.map
