import { __assign, __extends, __decorate, __metadata, __spread } from 'tslib';
import { Injector, ViewChild, ElementRef, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { NG_VALUE_ACCESSOR, FormsModule } from '@angular/forms';
import { PROP_STRING, PROP_ANY, PROP_NUMBER, PROP_BOOLEAN, register, registerFormWidget, styler, getOrderedDataset, provideAs, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { FormWidgetType, generateGUId, setCSS, noop } from '@wm/core';
import { DatasetAwareFormComponent } from '@wm/components/input';

var ratingProps = new Map([
    ['caption', PROP_STRING],
    ['class', PROP_STRING],
    ['datafield', PROP_STRING],
    ['dataset', PROP_ANY],
    ['datavaluesource', PROP_ANY],
    ['datavalue', PROP_STRING],
    ['displayexpression', PROP_STRING],
    ['displayfield', PROP_STRING],
    ['hint', PROP_STRING],
    ['iconcolor', PROP_STRING],
    ['iconsize', PROP_STRING],
    ['maxvalue', __assign({ value: 5 }, PROP_NUMBER)],
    ['name', PROP_STRING],
    ['orderby', PROP_STRING],
    ['readonly', PROP_BOOLEAN],
    ['show', __assign({ value: true }, PROP_BOOLEAN)],
    ['showcaptions', __assign({ value: true }, PROP_BOOLEAN)],
    ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
    ['usekeys', PROP_BOOLEAN]
]);
var registerProps = function () {
    register('wm-rating', ratingProps);
    registerFormWidget(FormWidgetType.RATING, new Map(ratingProps));
};

var DEFAULT_CLS = 'app-ratings';
var WIDGET_CONFIG = { widgetType: 'wm-rating', hostClass: DEFAULT_CLS };
var MAX_RATING = 10;
var DEFAULT_RATING = 5;
var RatingComponent = /** @class */ (function (_super) {
    __extends(RatingComponent, _super);
    function RatingComponent(inj) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        _this._id = generateGUId();
        styler(_this.nativeElement, _this);
        // prepare the rating options on dataset ready.
        var datasetSubscription = _this.dataset$.subscribe(function () {
            _this.prepareRatingDataset();
        });
        _this.registerDestroyListener(function () { return datasetSubscription.unsubscribe(); });
        // listen to changes in datavalue.
        var datavalueSubscription = _this.datavalue$.subscribe(function () { return _this.onDatavalueChange(_this.datavalue); });
        _this.registerDestroyListener(function () { return datavalueSubscription.unsubscribe(); });
        return _this;
    }
    RatingComponent_1 = RatingComponent;
    Object.defineProperty(RatingComponent.prototype, "selectedRatingValue", {
        get: function () {
            return this._selectedRatingValue;
        },
        set: function (val) {
            this._selectedRatingValue = val;
            this.calculateRatingsWidth();
        },
        enumerable: true,
        configurable: true
    });
    // Change event is registered from the template, Prevent the framework from registering one more event
    RatingComponent.prototype.handleEvent = function (node, eventName, eventCallback, locals) {
        if (eventName !== 'change' && eventName !== 'blur') {
            _super.prototype.handleEvent.call(this, this.ratingEl.nativeElement, eventName, eventCallback, locals);
        }
    };
    // This function returns the rating widget dataset containing the index, value and label
    RatingComponent.prototype.prepareRatingDataset = function () {
        if (!this.datasetItems.length) {
            this.ratingItems = [];
            this.caption = '';
        }
        var ratingItems = [];
        var data = this.datasetItems;
        var maxvalue = parseInt(this.maxvalue || this.datasetItems.length, 10);
        var maxValue = (maxvalue > MAX_RATING ? MAX_RATING : maxvalue) || DEFAULT_RATING;
        /**
         * 1. If datasetItems.length is more than maxValue (i.e. 10 ratings) then just extract maxValue of items from datasetItems.
         * 2. If datasetItems are not available then prepare ratings value depending on maxvalue. eg: 1,2,3 .. upto maxvalue
         * 3. If maxvalue / i value is more than datasetItems length, prepare default rating items for i values more than datasetItems.length
         */
        if (data.length && data.length > maxValue) {
            data = _.slice(data, 0, maxValue);
        }
        for (var i = maxValue; i > 0; i--) {
            if (!data.length) {
                ratingItems.push({ key: i, value: i, index: i, label: i });
            }
            else {
                if (i > data.length) {
                    ratingItems.push({ key: i, value: i, index: i, label: i });
                }
                else {
                    data = getOrderedDataset(data, 'index:desc');
                    ratingItems = ratingItems.concat(data);
                    break;
                }
            }
        }
        this.ratingItems = ratingItems;
        if (!data.length) { // constructs default datasetItems when there is no dataset binding.
            this.datasetItems = ratingItems;
        }
        this.onDatavalueChange(this.datavalue);
    };
    RatingComponent.prototype.onRatingClick = function ($event, rate) {
        this.modelByKey = rate.key;
        this.selectedRatingValue = rate.index;
        // support if the caption is binded in the old projects for backward compatibility
        if (!this.showcaptions) {
            this.caption = rate.label;
        }
        this.invokeOnTouched();
        // invoke on datavalue change.
        this.invokeOnChange(this.datavalue, $event || {}, true);
    };
    // Update the selected flag on datasetItems and assign the ratingValue.
    /**
     * On datavalue change, update the caption, selectedRatingValue.
     * 1. if datasetItems contain the selected item (check the selected flag on item), find the index of selected item.
     * 2. if not, just check if the datavalue is provided as the index on the item.
     *
     * @param dataVal datavalue
     */
    RatingComponent.prototype.onDatavalueChange = function (dataVal) {
        if (!_.isEmpty(this.datasetItems)) {
            var selectedItem = _.find(this.datasetItems, { 'selected': true });
            if (!selectedItem && !_.isUndefined(dataVal)) {
                selectedItem = _.find(this.datasetItems, function (item) {
                    return _.toString(item.index) === dataVal;
                });
                if (selectedItem) {
                    selectedItem.selected = true;
                }
            }
            if (!selectedItem) {
                // reset the  model if there is no item found.
                this.modelByKey = undefined;
                this.caption = '';
            }
            this.selectedRatingValue = selectedItem ? selectedItem.index : 0;
            if (selectedItem) {
                this.caption = selectedItem.label;
            }
        }
        else {
            this.selectedRatingValue = 0;
        }
        if (this.readonly) {
            // when dataset is not given but datavalue is provided which is integer
            if (!this.selectedRatingValue && !isNaN(dataVal)) {
                this.selectedRatingValue = (parseFloat(dataVal) <= this.maxvalue) ? dataVal : this.maxvalue;
            }
            this.ratingsWidth = this.calculateRatingsWidth(dataVal);
        }
    };
    RatingComponent.prototype.calculateRatingsWidth = function (dataVal) {
        var selectedRating = parseFloat(this.selectedRatingValue), starWidth = 0.925, maxValue = parseInt(this.maxvalue || this.datasetItems.length, 10) || DEFAULT_RATING;
        setCSS(this.nativeElement.querySelector('.ratings-container'), 'width', (starWidth * maxValue) + 'em');
        dataVal = dataVal || this.datavalue;
        if (dataVal === undefined || dataVal === '' || dataVal === null) {
            this.caption = '';
            return 0;
        }
        if (selectedRating <= maxValue && selectedRating >= 0) {
            return selectedRating * starWidth + 'em';
        }
        if (selectedRating > maxValue) {
            return maxValue * starWidth + 'em';
        }
    };
    RatingComponent.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'readonly') {
            if (nv) {
                this.ratingsWidth = this.calculateRatingsWidth();
            }
        }
        else if (key === 'maxvalue') {
            /** Storing the datavalue in temp variable and assiging back to datavalue
             * after default datalist is prepared to trigger Dataset Aware class select cycle
             * as the datavalue property change is triggering first rather than maxvalue
             * in the rating widget is used inside a datatable
             */
            var tempDataValue = this.datavalue;
            this.prepareRatingDataset();
            // reset all the items.
            this.resetDatasetItems();
            if (!_.isUndefined(tempDataValue)) {
                this.datavalue = tempDataValue;
            }
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    /* Detect touch enabled devices & update hover styles*/
    RatingComponent.prototype.onTouchStart = function ($event) {
        this.touchEnabled = true;
        this.onTouchStart = noop;
    };
    RatingComponent.prototype.onMouseleave = function () {
        this.caption = this.displayValue;
    };
    RatingComponent.prototype.onMouseOver = function ($event, rate) {
        this.caption = rate.label;
        !this.touchEnabled && !$event.target.classList.contains('rating-label-hover') && $event.target.classList.add('rating-label-hover');
    };
    var RatingComponent_1;
    RatingComponent.initializeProps = registerProps();
    RatingComponent.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    __decorate([
        ViewChild('ratingInput', /* TODO: add static flag */ { read: ElementRef }),
        __metadata("design:type", ElementRef)
    ], RatingComponent.prototype, "ratingEl", void 0);
    RatingComponent = RatingComponent_1 = __decorate([
        Component({
            selector: '[wmRating]',
            template: "<div [(ngModel)]=\"modelByKey\" (focus)=\"onFocus($event)\" ngDefaultControl role=\"radiogroup\" class=\"ratings-wrapper\">\n    <div *ngIf=\"!readonly\" class=\"rating-style\">\n        <label *ngFor=\"let rate of ratingItems;\"\n               [ngClass]=\"{active : rate.index <= selectedRatingValue}\"\n               [ngStyle]=\"{'font-size' :iconsize, color: rate.index <= selectedRatingValue && iconcolor}\"\n               [title]=\"rate.label || rate.index\"\n               (touchstart)=\"onTouchStart($event)\"\n               (mouseleave)=\"onMouseleave($event, rate)\"\n               (mouseover)=\"onMouseOver($event, rate)\"\n               [for]=\"_id + '-' + rate.index\">\n            <input #ratingInput type=\"radio\" aria-label=\"rating radio buttons\" aria-checked=\"false\" aria-multiselectable=\"true\"\n                   [id]=\"_id + '-' + rate.index\" (click)=\"onRatingClick($event, rate)\" name=\"ratings-id\" [value]=\"rate.key || rate.index\"/>\n        </label>\n    </div>\n    <div [class.hidden]=\"!readonly\" [ngStyle]=\"{'font-size' :iconsize}\" class=\"ratings-container disabled\" >\n        <div class=\"ratings active\" [ngStyle]=\"{width: ratingsWidth, color: iconcolor}\"></div>\n    </div>\n    <label *ngIf=\"showcaptions\" class=\"caption\" [textContent]=\"caption\"></label>\n</div>\n",
            providers: [
                provideAs(RatingComponent_1, NG_VALUE_ACCESSOR, true),
                provideAsWidgetRef(RatingComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], RatingComponent);
    return RatingComponent;
}(DatasetAwareFormComponent));

var components = [
    RatingComponent
];
var RatingModule = /** @class */ (function () {
    function RatingModule() {
    }
    RatingModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                FormsModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], RatingModule);
    return RatingModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { RatingModule, RatingComponent as ɵa, registerProps as ɵb };
//# sourceMappingURL=index.js.map
