import { __decorate, __metadata } from 'tslib';
import { Injector, Directive, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { register, PROP_BOOLEAN, PROP_STRING, StylableComponent, updateDeviceView, provideAsWidgetRef, styler, PROP_NUMBER, APPLY_STYLES_TYPE, PullToRefresh, WmComponentsModule } from '@wm/components/base';
import { Title } from '@angular/platform-browser';
import { EventNotifier, Viewport, isMobileApp, switchClass, App } from '@wm/core';

const registerProps = () => {
    register('wm-page', new Map([
        ['cache', Object.assign({ value: false }, PROP_BOOLEAN)],
        ['refreshdataonattach', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['pagetitle', PROP_STRING]
    ]));
};

var PageDirective_1;
const DEFAULT_CLS = 'app-page container';
const WIDGET_CONFIG = { widgetType: 'wm-page', hostClass: DEFAULT_CLS };
let PageDirective = PageDirective_1 = class PageDirective extends StylableComponent {
    constructor(inj, titleService, viewport) {
        super(inj, WIDGET_CONFIG);
        this.titleService = titleService;
        this.viewport = viewport;
        this._eventNotifier = new EventNotifier(false);
        this.refreshdataonattach = true;
    }
    onPropertyChange(key, nv, ov) {
        if (key === 'pagetitle') {
            this.titleService.setTitle(nv);
        }
        else {
            super.onPropertyChange(key, nv, ov);
        }
    }
    /**
     * A child component can notify page using this method. Notified event will be passed to
     * subscribed children only after page initialization.
     *
     * @param {string} eventName
     * @param data
     */
    notify(eventName, ...data) {
        this._eventNotifier.notify.apply(this._eventNotifier, arguments);
    }
    /**
     * The main purpose of this function is to provide communication between page children objects.
     * Child component can subscribe for an event that will be emitted by another child component.
     *
     * @param eventName
     * @param {(data: any) => void} callback
     * @returns {any}
     */
    subscribe(eventName, callback) {
        return this._eventNotifier.subscribe(eventName, callback);
    }
    ngAfterViewInit() {
        setTimeout(() => {
            this._eventNotifier.start();
            updateDeviceView(this.nativeElement, this.getAppInstance().isTabletApplicationType);
        }, 1);
        this.registerDestroyListener(this.viewport.subscribe("resize" /* RESIZE */, args => {
            this.invokeEventCallback('resize', { $event: args.$event, widget: this, data: args.data });
        }));
        this.registerDestroyListener(this.viewport.subscribe("orientationchange" /* ORIENTATION_CHANGE */, args => {
            this.invokeEventCallback('orientationchange', { $event: args.$event, widget: this, data: args.data });
        }));
    }
    ngOnAttach() {
        this.titleService.setTitle(this.pagetitle);
        this.invokeEventCallback('attach', { widget: this });
        this.notify('attach', {
            refreshData: this.refreshdataonattach
        });
    }
    ngOnDetach() {
        this.invokeEventCallback('detach', { widget: this });
        this.notify('detach');
    }
    ngOnDestroy() {
        this.invokeEventCallback('destroy', { widget: this });
        this._eventNotifier.destroy();
    }
};
PageDirective.initializeProps = registerProps();
PageDirective.ctorParameters = () => [
    { type: Injector },
    { type: Title },
    { type: Viewport }
];
PageDirective = PageDirective_1 = __decorate([
    Directive({
        selector: '[wmPage]',
        providers: [
            provideAsWidgetRef(PageDirective_1)
        ]
    }),
    __metadata("design:paramtypes", [Injector, Title, Viewport])
], PageDirective);

const registerProps$1 = () => {
    register('wm-content', new Map([
        ['name', PROP_STRING],
        ['class', PROP_STRING]
    ]));
};

var ContentComponent_1;
const DEFAULT_CLS$1 = 'app-content clearfix';
const WIDGET_CONFIG$1 = { widgetType: 'wm-content', hostClass: DEFAULT_CLS$1 };
let ContentComponent = ContentComponent_1 = class ContentComponent extends StylableComponent {
    constructor(inj) {
        super(inj, WIDGET_CONFIG$1);
        styler(this.nativeElement, this);
    }
};
ContentComponent.initializeProps = registerProps$1();
ContentComponent.ctorParameters = () => [
    { type: Injector }
];
ContentComponent = ContentComponent_1 = __decorate([
    Component({
        selector: '[wmContent]',
        template: "<div class=\"row app-content-row clearfix\">\n    <ng-content></ng-content>\n</div>",
        providers: [
            provideAsWidgetRef(ContentComponent_1)
        ]
    }),
    __metadata("design:paramtypes", [Injector])
], ContentComponent);

const registerProps$2 = () => {
    register('wm-page-content', new Map([
        ['class', PROP_STRING],
        ['columnwidth', PROP_NUMBER],
        ['content', PROP_STRING],
        ['name', PROP_STRING],
        ['pulltorefresh', Object.assign({ value: isMobileApp() }, PROP_BOOLEAN)]
    ]));
};

var PageContentComponent_1;
const DEFAULT_CLS$2 = 'app-page-content app-content-column';
const WIDGET_CONFIG$2 = { widgetType: 'wm-page-content', hostClass: DEFAULT_CLS$2 };
let PageContentComponent = PageContentComponent_1 = class PageContentComponent extends StylableComponent {
    constructor(inj, app) {
        super(inj, WIDGET_CONFIG$2);
        this.app = app;
        styler(this.nativeElement, this, APPLY_STYLES_TYPE.CONTAINER);
        this.registerDestroyListener(this.app.subscribe('pullToRefresh:enable', () => {
            this.childPullToRefresh = true;
            this.initPullToRefresh();
        }));
    }
    onPropertyChange(key, nv, ov) {
        if (key === 'columnwidth') {
            switchClass(this.nativeElement, `col-md-${nv} col-sm-${nv}`, `col-md-${ov} col-sm-${ov}`);
        }
        else if (key === 'pulltorefresh' && nv) {
            // creating instance after timeout as the smoothscroll styles where getting added on pull refresh-container
            setTimeout(() => {
                this.initPullToRefresh();
            });
        }
        else {
            super.onPropertyChange(key, nv, ov);
        }
    }
    // when list component is ready, pulltorefresh instance is created and this appends pullToRefresh element on the page content.
    initPullToRefresh() {
        const hasPullToRefreshEvent = this.hasEventCallback('pulltorefresh');
        if (!this.pullToRefreshIns && (this.childPullToRefresh || hasPullToRefreshEvent) && this.pulltorefresh) {
            this.pullToRefreshIns = new PullToRefresh($(this.nativeElement), this.app, () => {
                if (hasPullToRefreshEvent) {
                    this.invokeEventCallback('pulltorefresh');
                }
                else {
                    this.app.notify('pulltorefresh');
                }
            });
            this.registerDestroyListener(() => {
                if (this.pullToRefreshIns.cancelSubscription) {
                    this.pullToRefreshIns.cancelSubscription();
                }
            });
        }
    }
};
PageContentComponent.initializeProps = registerProps$2();
PageContentComponent.ctorParameters = () => [
    { type: Injector },
    { type: App }
];
PageContentComponent = PageContentComponent_1 = __decorate([
    Component({
        selector: '[wmPageContent]',
        template: "<ng-content></ng-content>",
        providers: [
            provideAsWidgetRef(PageContentComponent_1)
        ]
    }),
    __metadata("design:paramtypes", [Injector, App])
], PageContentComponent);

const components = [
    PageDirective,
    ContentComponent,
    PageContentComponent
];
let PageModule = class PageModule {
};
PageModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], PageModule);

/**
 * Generated bundle index. Do not edit.
 */

export { PageDirective, PageModule, registerProps as ɵa, ContentComponent as ɵb, registerProps$1 as ɵc, PageContentComponent as ɵd, registerProps$2 as ɵe };
//# sourceMappingURL=index.js.map
