import { __extends, __read, __decorate, __metadata } from 'tslib';
import { Injectable, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { HttpHeaders, HttpParams, HttpRequest, HttpClient, HttpResponse, HttpClientModule } from '@angular/common/http';
import { getValidJSON, replace, AbstractHttpService } from '@wm/core';
import { Subject } from 'rxjs';

var HttpServiceImpl = /** @class */ (function (_super) {
    __extends(HttpServiceImpl, _super);
    function HttpServiceImpl(httpClient) {
        var _this = _super.call(this) || this;
        _this.httpClient = httpClient;
        _this.nonBodyTypeMethods = ['GET', 'DELETE', 'HEAD', 'OPTIONS', 'JSONP'];
        _this.sessionTimeoutObservable = new Subject();
        _this.sessionTimeoutQueue = [];
        return _this;
    }
    /**
     * This method handles session timeout.
     * @param options
     * @param resolve
     * @param reject
     */
    HttpServiceImpl.prototype.handleSessionTimeout = function (options, subscriber401) {
        this.sessionTimeoutQueue.push({
            requestInfo: options,
            sub401: subscriber401
        });
        this.on401();
    };
    /**
     * Generates a request with provided options
     * @param options, request params/options
     */
    HttpServiceImpl.prototype.generateRequest = function (options) {
        var reqHeaders = new HttpHeaders(), reqParams = new HttpParams();
        var headers = options.headers;
        var params = options.params;
        var responseType = options.responseType;
        var withCredentials = options.withCredentials;
        // this header is not to be sent with non-proxy calls from service variable
        if (!options.isDirectCall) {
            reqHeaders = reqHeaders.append('X-Requested-With', 'XMLHttpRequest');
        }
        // headers
        if (headers) {
            Object.entries(headers).forEach(function (_a) {
                var _b = __read(_a, 2), k = _b[0], v = _b[1];
                return reqHeaders = reqHeaders.append(k, v);
            });
        }
        // params
        if (params) {
            Object.entries(params).forEach(function (_a) {
                var _b = __read(_a, 2), k = _b[0], v = _b[1];
                return reqParams = reqParams.append(k, v);
            });
        }
        var third, fourth;
        var reqOptions = {
            headers: reqHeaders,
            params: reqParams,
            responseType: responseType,
            withCredentials: withCredentials
        };
        // Even if method is not expecting request body, if body is passed in request, pass it on.
        if (_.includes(this.nonBodyTypeMethods, options.method && options.method.toUpperCase()) && options.data === undefined) {
            third = reqOptions;
            fourth = null;
        }
        else {
            third = options.data;
            fourth = reqOptions;
        }
        return new HttpRequest(options.method, options.url, third, fourth);
    };
    /**
     * This method filters and returns error message from the failed network call response.
     * @param err, error form network call failure
     */
    HttpServiceImpl.prototype.getErrMessage = function (err) {
        var HTTP_STATUS_MSG = {
            404: this.getLocale()['MESSAGE_404_ERROR'] || 'Requested resource not found',
            401: this.getLocale()['MESSAGE_401_ERROR'] || 'Requested resource requires authentication',
            403: this.getLocale()['LABEL_FORBIDDEN_MESSAGE'] || 'The requested resource access/action is forbidden.'
        };
        // check if error message present for responded http status
        var errMsg = HTTP_STATUS_MSG[err.status];
        var errorDetails = err.error;
        errorDetails = getValidJSON(errorDetails) || errorDetails;
        // WM services have the format of error response as errorDetails.error
        if (errorDetails && errorDetails.errors) {
            errMsg = this.parseErrors(errorDetails.errors) || errMsg || 'Service Call Failed';
        }
        else {
            errMsg = errMsg || 'Service Call Failed';
        }
        return errMsg;
    };
    /**
     * Make a http call and returns an observable that can be cancelled
     * @param options, options using which the call needs to be made
     */
    HttpServiceImpl.prototype.sendCallAsObservable = function (options) {
        var req = this.generateRequest(options);
        return this.httpClient.request(req);
    };
    /**
     * Makes a http call and return a promise
     * @param options, options using which the call needs to be made
     */
    HttpServiceImpl.prototype.send = function (options) {
        var _this = this;
        var req = this.generateRequest(options);
        return new Promise(function (resolve, reject) {
            _this.httpClient.request(req).toPromise().then(function (response) {
                resolve(response);
            }, function (err) {
                if (_this.isPlatformSessionTimeout(err)) {
                    err._401Subscriber.asObservable().subscribe(function (response) {
                        resolve(response);
                    }, function (e) {
                        reject(e);
                    });
                }
                else {
                    var errMsg = _this.getErrMessage(err);
                    reject({
                        error: errMsg,
                        details: err
                    });
                }
            });
        });
    };
    HttpServiceImpl.prototype.setLocale = function (locale) {
        this.localeObject = locale;
    };
    HttpServiceImpl.prototype.getLocale = function () {
        return this.localeObject;
    };
    HttpServiceImpl.prototype.parseErrors = function (errors) {
        var _this = this;
        var errMsg = '';
        if (errors && errors.error && errors.error.length) {
            errors.error.forEach(function (errorDetails, i) {
                errMsg += _this.parseError(errorDetails) + (i > 0 ? '\n' : '');
            });
        }
        return errMsg;
    };
    HttpServiceImpl.prototype.parseError = function (errorObj) {
        var errMsg;
        errMsg = errorObj.message ? replace(errorObj.message, errorObj.parameters, true) : ((errorObj.parameters && errorObj.parameters[0]) || '');
        return errMsg;
    };
    HttpServiceImpl.prototype.getHeader = function (error, headerKey) {
        return error.headers.get(headerKey);
    };
    HttpServiceImpl.prototype.isPlatformSessionTimeout = function (error) {
        var MSG_SESSION_NOT_FOUND = 'Session Not Found';
        return error.status === 401 && this.getHeader(error, 'x-wm-login-errormessage') === MSG_SESSION_NOT_FOUND;
    };
    HttpServiceImpl.prototype.get = function (url, options) {
        options = options || {};
        options.url = url;
        options.method = 'get';
        return this.send(options).then(function (response) { return response.body; });
    };
    HttpServiceImpl.prototype.post = function (url, data, options) {
        options = options || {};
        options.url = url;
        options.method = 'post';
        options.data = data;
        return this.send(options);
    };
    HttpServiceImpl.prototype.put = function (url, data, options) {
        options = options || {};
        options.url = url;
        options.method = 'put';
        options.data = data;
        return this.send(options);
    };
    HttpServiceImpl.prototype.patch = function (url, data, options) {
        options = options || {};
        options.url = url;
        options.method = 'patch';
        options.data = data;
        return this.send(options);
    };
    HttpServiceImpl.prototype.delete = function (url, options) {
        options = options || {};
        options.url = url;
        options.method = 'delete';
        return this.send(options);
    };
    HttpServiceImpl.prototype.head = function (url, options) {
        options = options || {};
        options.url = url;
        options.method = 'head';
        return this.send(options);
    };
    HttpServiceImpl.prototype.jsonp = function (url, options) {
        options = options || {};
        options.url = url;
        options.method = 'jsonp';
        return this.send(options);
    };
    HttpServiceImpl.prototype.upload = function (url, data, options) {
        var req = new HttpRequest('POST', url, data, {
            reportProgress: true // for progress data
        });
        return this.httpClient.request(req);
        // return this.httpClient.post(url, data, {
        //     reportProgress: true
        // });
    };
    /**
     * registers a callback to be trigerred on session timeout
     * @param callback
     */
    HttpServiceImpl.prototype.registerOnSessionTimeout = function (callback) {
        this.sessionTimeoutObservable.asObservable().subscribe(callback);
    };
    /**
     * trigger the registered methods on session timeout
     */
    HttpServiceImpl.prototype.on401 = function () {
        this.sessionTimeoutObservable.next();
    };
    HttpServiceImpl.prototype.pushToSessionFailureQueue = function (callback) {
        this.sessionTimeoutQueue.push({
            callback: callback
        });
    };
    /**
     * Execute queued requests, failed due to session timeout
     */
    HttpServiceImpl.prototype.executeSessionFailureRequests = function () {
        var queue = this.sessionTimeoutQueue;
        var that = this;
        that.sessionTimeoutQueue = [];
        queue.forEach(function (data) {
            if (_.isFunction(data.callback)) {
                data.callback();
            }
            else {
                data.requestInfo.headers.headers.delete('x-wm-xsrf-token');
                that.httpClient.request(data.requestInfo)
                    .subscribe(function (response) {
                    if (response && response.type && data.sub401) {
                        data.sub401.next(response);
                    }
                }, function (reason) {
                    data.sub401.error(reason);
                });
            }
        });
    };
    HttpServiceImpl.ctorParameters = function () { return [
        { type: HttpClient }
    ]; };
    HttpServiceImpl = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [HttpClient])
    ], HttpServiceImpl);
    return HttpServiceImpl;
}(AbstractHttpService));

/**
 * This class creates a custom WmHttpRequest.
 */
var WmHttpRequest = /** @class */ (function () {
    function WmHttpRequest() {
        this.headers = [];
    }
    /**
     * This method converts the given wm HttpRequest to angular HttpRequest
     * @param reqObj, the wm HttpRequest
     */
    WmHttpRequest.prototype.wmToAngularRequest = function (reqObj) {
        var angReq;
        angReq = new HttpRequest(reqObj.method, reqObj.url);
        angReq = angReq.clone();
        angReq.body = reqObj.body;
        angReq.method = reqObj.method;
        angReq.responseType = reqObj.responseType;
        angReq.reportProgress = reqObj.reportProgress;
        angReq.urlWithParams = reqObj.url;
        angReq.withCredentials = reqObj.withCredentials;
        reqObj.headers.forEach(function (value, header) {
            angReq.headers = angReq.headers.append(header, value);
        });
        return angReq;
    };
    /**
     * This method converts the given angular HttpRequest to wm HttpRequest
     * @param req, the angular HttpRequest
     */
    WmHttpRequest.prototype.angularToWmRequest = function (req) {
        var wmReq = new WmHttpRequest();
        var headerMap = new Map();
        wmReq.body = req.body;
        wmReq.method = req.method;
        wmReq.responseType = req.responseType;
        wmReq.reportProgress = req.reportProgress;
        wmReq.url = req.urlWithParams;
        wmReq.withCredentials = req.withCredentials;
        req.headers.keys().forEach(function (header) {
            headerMap.set(header, req.headers.headers.get(header.toLowerCase())[0]);
        });
        wmReq.headers = headerMap;
        return wmReq;
    };
    return WmHttpRequest;
}());

/**
 * This class creates a custom WmHttpRequest.
 */
var WmHttpResponse = /** @class */ (function (_super) {
    __extends(WmHttpResponse, _super);
    function WmHttpResponse() {
        return _super.call(this) || this;
    }
    /**
     * This method converts the given wm HttpResponse to angular HttpResponse
     * @param respObj, the wm HttpResponse
     */
    WmHttpResponse.prototype.wmToAngularResponse = function (respObj) {
        var angResp;
        angResp = new HttpResponse();
        angResp = angResp.clone();
        angResp.body = respObj.body;
        angResp.error = respObj.error;
        angResp.message = respObj.message;
        angResp.url = respObj.url;
        angResp.ok = respObj.ok;
        angResp.status = respObj.status;
        angResp.statusText = respObj.statusText;
        respObj.headers.forEach(function (value, header) {
            angResp.headers = angResp.headers.append(header, value);
        });
        return angResp;
    };
    /**
     * This method converts the given wm HttpResponse to angular HttpResponse
     * @param respObj, the angular HttpResponse
     */
    WmHttpResponse.prototype.angularToWmResponse = function (respObj) {
        var wmResp;
        var headerMap = new Map();
        wmResp = new WmHttpResponse();
        wmResp.body = respObj.body;
        wmResp.error = respObj.error;
        wmResp.message = respObj.message;
        wmResp.url = respObj.url;
        wmResp.ok = respObj.ok;
        wmResp.status = respObj.status;
        wmResp.statusText = respObj.statusText;
        respObj.headers.keys().forEach(function (header) {
            headerMap.set(header, respObj.headers.headers.get(header.toLowerCase())[0]);
        });
        wmResp.headers = headerMap;
        return wmResp;
    };
    return WmHttpResponse;
}(WmHttpRequest));

var HttpServiceModule = /** @class */ (function () {
    function HttpServiceModule() {
    }
    HttpServiceModule.forRoot = function () {
        return {
            ngModule: CommonModule,
            providers: [
                { provide: AbstractHttpService, useClass: HttpServiceImpl }
            ]
        };
    };
    HttpServiceModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                HttpClientModule
            ],
            declarations: []
        })
    ], HttpServiceModule);
    return HttpServiceModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { HttpServiceImpl, HttpServiceModule, WmHttpRequest, WmHttpResponse };
//# sourceMappingURL=index.js.map
